package no.ntnu.master.form;

import com.badlogic.gdx.math.Circle;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.utils.Array;

import java.security.SecureRandom;
import java.util.Random;

import no.ntnu.master.objects.LineSegment;
import no.ntnu.master.utils.Enums;
import no.ntnu.master.utils.Psychophysics;
import no.ntnu.master.utils.Settings;
import no.ntnu.master.utils.World;

/**
 * Created by B on 22.12.2015.
 * World for the form portion of the application
 */
public class FormWorld implements World {

    private Enums.WorldStates currentState;

    private boolean isFixed;

    private Random random;

    private float runningTime;

    public float coherencePercent;
    private float maxRunningTime;

    private String coherentPatchSide;

    public float patchGap;
    public Rectangle rectLeft;
    public Rectangle rectRight;

    public Array<LineSegment> segments;

    //TODO Remove these before final version
    public int correctAnswerCounter = 0;
    public int wrongAnswerCounter = 0;
    // --Commented out by Inspection (18.02.2016 23:03):public int reversalCounter = 0;

    public FormWorld(boolean isFixed) {
        this.isFixed = isFixed;
        currentState = Enums.WorldStates.PAUSED;
        random = new SecureRandom();
        runningTime = 0;

        coherentPatchSide = "Right";

        coherencePercent = Settings.getFormCoherency();
        if (isFixed) {
            maxRunningTime = Settings.getFormFDetectionTime();
        } else maxRunningTime = Settings.getFormRDetectionTime();


        segments = new Array<>();

        createPatches();

        createLineSegments();
    }

    private void createPatches() {
        patchGap = Psychophysics.getPatchSeparationInPixels();
        float patchWidth = Psychophysics.getPatchWidthInPixels();
        float patchHeight = Psychophysics.getPatchHeightInPixels();

        float screenXCenter = Settings.getScreenWidthPixels() / 2;
        float screenYCenter = Settings.getScreenHeightPixels() / 2;

        rectLeft = new Rectangle(screenXCenter-patchWidth-(patchGap /2), screenYCenter-(patchHeight/2), patchWidth, patchHeight);
        rectRight = new Rectangle(rectLeft.x+patchWidth+ patchGap, screenYCenter-(patchHeight/2), patchWidth ,patchHeight);
    }

    /**
     * Calculates and fetches parameters for creating segments
     * Passes parameters {@link #autoMode(float, float, float)}
     * or {@link #manualMode(float, float, float, float, float, float)}
     * dependent on the flag set in {@link no.ntnu.master.settingcreens.FormSettings#autoCheckBox}
     */
    private void createLineSegments() {
        float x;
        float y;

        float d = Psychophysics.visualAngleInPixels(
                Settings.getFormDiameterWb(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenWidthPixels(),
                Settings.getScreenWidth()
        );

        if(!random.nextBoolean()) {
            coherentPatchSide = "Left";
            if (isFixed) {
                x = rectLeft.x+(rectLeft.width / 2);
                y = rectLeft.y+(rectLeft.height / 2);
            } else {
                float xMax = (rectLeft.x+rectLeft.width)-(d/2);
                float xMin = (rectLeft.x) + (d/2);
                float yMax = (rectLeft.y+rectLeft.height) - (d/2);
                float yMin = (rectLeft.y) + (d/2);
                x = random.nextFloat() * (xMax-xMin) + xMin;
                y = random.nextFloat() * (yMax-yMin) + yMin;
            }

        } else {
            coherentPatchSide = "Right";
            if (isFixed) {
                x = rectRight.x+(rectRight.width/2);
                y = rectRight.y+(rectRight.height / 2);
            } else {
                float xMax = (rectRight.x+rectRight.width)-(d/2);
                float xMin = (rectRight.x) + (d/2);
                float yMax = (rectRight.y+rectRight.height) - (d/2);
                float yMin = (rectRight.y) + (d/2);
                x = random.nextFloat() * (xMax-xMin) + xMin;
                y = random.nextFloat() * (yMax-yMin) + yMin;
            }
        }

        float circleGap = Psychophysics.visualAngleInPixels(
                Settings.getFormCirclesGap(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenWidthPixels(),
                Settings.getScreenWidth()
        );
        float lineGapOffset = Psychophysics.visualAngleInPixels(
                Settings.getFormLineGap(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenWidthPixels(),
                Settings.getScreenWidth()
        );
        float lineLength = Psychophysics.visualAngleInPixels(
                Settings.getFormLineLength(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenWidthPixels(),
                Settings.getScreenWidth()
        );
        float r = (d/2f);
        if (!Settings.getFormAutoMode()) {
            manualMode(x, y, r, lineLength, lineGapOffset, circleGap);
        } else {
            autoMode(x, y, r);
        }
    }

    /**
     * Creates a set{@see no.ntnu.master.settingscreens.FormSettings}
     * number of concentric circles with the line segments aligned to the tangent of a given circle
     * with center x,y and max radius of r. When coherence is lowered, segments of the circle is
     * relocated at a random spot within the same patch. Noise segments are created at random.
     * @param x The x coordinate of the given circle.
     * @param y The y coordinate of the given circle.
     * @param r The radius of the given circle.
     * @param lineLength The length of each line in the concentric circles.
     * @param lineGapOffset The space between each line in the same circle.
     * @param circleGap The space between each concentric circle.
     */
    private void manualMode(float x, float y, float r,
                            float lineLength, float lineGapOffset, float circleGap) {
        int lCounter = 0;
        int rCounter = 0;
        for (int i = 0; i < Settings.getFormNrOfCircles(); i++) {
            int start = random.nextInt(360);
            float maxLines = (float) (Math.PI * (r * 2)) / (lineLength + lineGapOffset);
            float gap = Math.round((360 / maxLines));
            while (360 % gap != 0 && gap > 0) {
                gap--;
            }
            if (gap <= 0) {
                gap = 1;
            }

            for (int j = start; j < 360 + start; j += gap) {
                float tempX = (float) (x + Math.sin(Math.toRadians(j)) * r);
                float tempY = (float) (y + Math.cos(Math.toRadians(j)) * r);
                float angle = (360 - j);
                segments.add(new LineSegment(tempX, tempY, angle, 0));
            }
            r -= circleGap;
        }
        for (int i = 0; i < (segments.size - (segments.size * (coherencePercent / 100))); i++) {
            float tempX;
            float tempY;
            if(coherentPatchSide.equals("Left")) {
                tempX = random.nextFloat() * ((rectLeft.x + rectLeft.width) - rectLeft.x) + rectLeft.x;
                tempY = random.nextFloat() * ((rectLeft.y + rectLeft.height) - rectLeft.y) + rectLeft.y;
            }
            else {
                tempX = random.nextFloat() * ((rectRight.x + rectLeft.width) - rectRight.x) + rectRight.x;
                tempY = random.nextFloat() * ((rectRight.y + rectRight.height) - rectRight.y) + rectRight.y;
            }
            segments.get(i).setPosition(tempX, tempY);
            segments.get(i).setAngleOffset(random.nextFloat() * (360f - 0f) + 0f);
        }

        if (coherentPatchSide.equals("Left")) {
            lCounter = segments.size;
        } else rCounter = segments.size;

        while (segments.size < Settings.getFormMaxLineSegments()*2) {
            if ((lCounter) < Settings.getFormMaxLineSegments()) {
                float xL = random.nextFloat() * ((rectLeft.x + rectLeft.width) - rectLeft.x) + rectLeft.x;
                float yL = random.nextFloat() * ((rectLeft.y + rectLeft.height) - rectLeft.y) + rectLeft.y;
                float angleL = random.nextFloat() * (360f - 0f) + 0f;
                segments.add(new LineSegment(xL, yL, angleL, 0f));
                lCounter++;
            }

            if ((rCounter) < Settings.getFormMaxLineSegments()) {
                float xR = random.nextFloat() * ((rectRight.x + rectLeft.width) - rectRight.x) + rectRight.x;
                float yR = random.nextFloat() * ((rectRight.y + rectRight.height) - rectRight.y) + rectRight.y;
                float angleR = random.nextFloat() * (360f - 0f) + 0f;
                segments.add(new LineSegment(xR, yR, angleR, 0f));
                rCounter++;
            }
        }
    }

    /**
     * Creates equally distributed line segments in both patches. Both patches will look the same
     * at 0 percent coherence.
     * Line segments within the confines of the given circle are angled to the tangent of the circle.
     * @param x The x coordinate of the given circle.
     * @param y The y coordinate of the given circle.
     * @param r The radius of the given circle.
     */
    private void autoMode(float x, float y, float r) {
        Circle circle = new Circle(x, y, r);
        float h = rectLeft.height;
        float w = rectLeft.width;
        float area = h*w;
        float pointArea = area/Settings.getFormMaxLineSegments();
        float length = (float)Math.sqrt(pointArea);

        for (float i = length/2;i<w;i+=length) {
            for (float j=length/2;j<h;j+=length) {
                float angle = random.nextFloat() * (360) + 0;
                segments.add(new LineSegment(rectLeft.x+i, rectLeft.y+j, angle, 0));
                segments.add(new LineSegment(rectRight.x+i, rectRight.y+j, angle, 0));
            }
        }

        Array<LineSegment> circleSegments = new Array<>();
        for(int i=0;i<segments.size;i++) {
            LineSegment seg = segments.get(i);
            if(circle.contains(seg.getX(), seg.getY())) {
                circleSegments.add(segments.removeIndex(i));
            }
        }

        for (int i=1;i<circleSegments.size;i++) {
            for (int j=i-1;
                 (j>=0 && circleSegments.get(j+1).getPosition().dst(x, y) > circleSegments.get(j).getPosition().dst(x, y));
                 j--) {
                    circleSegments.swap(j+1, j);
            }
        }

        //The center of the concentric circle will not be a circle
        for (int i=circleSegments.size-4;i>(circleSegments.size-(circleSegments.size*(coherencePercent/100)));i--) {
            LineSegment seg = circleSegments.get(i);
            float angle = (float) Math.toDegrees(Math.atan2(circle.y-seg.getY(), circle.x-seg.getX()));
            angle+=90;
            seg.setAngle(angle);
        }

        segments.addAll(circleSegments);
        circleSegments.clear();
    }

    public void update(float delta) {
//        reversalCounter = InputHandler.getReversalCounter();
        switch (currentState) {
            case FINISHED: break;
            case PAUSED: paused(); break;
            case RUNNING: running(delta); break;
        }
    }

    private void paused() {
        if (runningTime >= maxRunningTime) {
            runningTime = 0;
        }
    }

    private void running(float delta) {
        runningTime+= delta;
        if (runningTime >= maxRunningTime) {
            currentState = Enums.WorldStates.PAUSED;
        }
    }

    @Override
    public Enums.WorldStates getState() {
        return currentState;
    }

    @Override
    public void setState(Enums.WorldStates worldState) {
        this.currentState = worldState;
    }

    @Override
    public String getCoherentPatch() {
        return coherentPatchSide;
    }

    @Override
    public float getCoherencyPercent() { return coherencePercent; }

    @Override
    public boolean getIsFixed() { return isFixed; }

    @Override
    public void updateCoherency(float factor, boolean isCorrectAnswer) {
        float temp = coherencePercent * factor;

        //Update coherency and counters
        if (isCorrectAnswer) {
            if (factor > 1) {
                temp -= coherencePercent;
                coherencePercent -= temp;
            } else coherencePercent = temp;
            correctAnswerCounter++;
        } else {
            if (temp > 100) {
                coherencePercent = 100;
            } else coherencePercent = temp;
            wrongAnswerCounter++;
        }
        //Create line segments
        segments.clear();
        createLineSegments();
        runningTime = 0;
    }
}
