package no.ntnu.master.menus;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.screens.FormScreen;
import no.ntnu.master.screens.MotionScreen;
import no.ntnu.master.utils.AssetLoader;


/**
 * Created by B on 16.07.2015.
 * The menu screen
 */
public class MainMenu extends ScreenAdapter {

    /**
     * Reference to the game object used for changing screens
     */
    private DyslexiaDetection game;

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;
    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;

    /**
     * Variables used to create the user interface
     */
    private TextButton buttonMot, buttonFormF, buttonFormR, buttonSettings, buttonExit;

    /**
     * A stage and table for holding the various user interface elements
     */
    private Stage stage;
    private Table table;

    /**
     * Button text
     */
    private final String BUTTON_MOT_TEXT = "Motion test";
    private final String BUTTON_FORMF_TEXT = "Form fixed test";
    private final String BUTTON_FORMR_TEXT = "Form random test";
    private final String BUTTON_SETTINGS_TEXT = "Settings";
    private final String BUTTON_EXIT_TEXT = "Exit";

    /**
     * Creates a new Menu screen
     * @param game Reference to the game object
     */
    public MainMenu(DyslexiaDetection game) {
        this.game = game;
        initMenuScreen();
        initUiElements();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initMenuScreen() {
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create and fill the stage with the gui elements
     */
    private void initUiElements() {
        buttonMot = new TextButton(BUTTON_MOT_TEXT, uiSkin);
        buttonMot.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                game.setScreen(new MotionScreen(game));
            }
        });


        buttonFormF = new TextButton(BUTTON_FORMF_TEXT, uiSkin);
        buttonFormF.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                game.setScreen(new FormScreen(game, true));
            }
        });

        buttonFormR = new TextButton(BUTTON_FORMR_TEXT, uiSkin);
        buttonFormR.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                game.setScreen(new FormScreen(game, false));
            }
        });

        buttonSettings = new TextButton(BUTTON_SETTINGS_TEXT, uiSkin);
        buttonSettings.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                game.setScreen(new SettingsMenu(game));
            }
        });

        buttonExit = new TextButton(BUTTON_EXIT_TEXT, uiSkin);
        buttonExit.setColor(Color.valueOf("FF3333"));
        buttonExit.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.app.exit();
            }
        });

        table.setX(viewport.getScreenWidth() / 2);
        table.setY(viewport.getScreenHeight() / 2);

        table.add(buttonMot).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonFormF).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonFormR).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonSettings).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonExit).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();

        stage.addActor(table);
    }

    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK)) {
            if (!game.backPushed) {
                Gdx.app.exit();
            }
        } else if(game.backPushed) {
            game.backPushed = false;
        }
        stage.act();
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(width / 2);
        table.setY(height / 2);
    }

    @Override
    public void pause() {

    }

    @Override
    public void resume() {

    }

    @Override
    public void hide() {

    }

    @Override
    public void dispose() {
        super.dispose();
        uiSkin.dispose();
    }
}
