package no.ntnu.master.menus;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Dialog;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.settingcreens.FormSettings;
import no.ntnu.master.settingcreens.InputSettings;
import no.ntnu.master.settingcreens.MotionSettings;
import no.ntnu.master.settingcreens.ScreenSettings;
import no.ntnu.master.settingcreens.StaircaseSettings;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.Settings;

/**
 * Created by B on 06.01.2016.
 * Menu for choosing the different settings available
 */
public class SettingsMenu extends ScreenAdapter {
    /**
     * Reference to the game object used for changing screens
     */
    private DyslexiaDetection game;

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;
    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;


    /**
     * A stage and table for holding the various user interface elements
     */
    private Stage stage;
    private Table table;

    /**
     * Buttons
     */
    private TextButton buttonMotionSettings, buttonFormSettings, buttonScreenSettings,
            buttonStaircaseSettings, buttonInputSettings ,buttonBack, buttonReset;

    /**
     * Button text
     */
    private final String BUTTON_MOT_TEXT = "Motion Settings";
    private final String BUTTON_FORM_TEXT = "Form Settings";
    private final String BUTTON_SCREEN_TEXT = "Screen Settings";
    private final String BUTTON_STAIR_TEXT = "Staircase Settings";
    private final String BUTTON_INPUT_TEXT = "Input Settings";
    private final String BUTTON_RESET_TEXT = "Reset all settings";
    private final String BUTTON_BACK_TEXT = "Main Menu";

    /**
     * Dialog for resetting settings
     */
    private Dialog dialog;
    private boolean dialogIsShown = false;


    public SettingsMenu(DyslexiaDetection game) {
        this.game = game;
        initMenuScreen();
        initUiElements();
        buildUi();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initMenuScreen() {
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create and fill the stage with the gui elements
     */
    private void initUiElements() {
        buttonMotionSettings = new TextButton(BUTTON_MOT_TEXT, uiSkin);
        buttonMotionSettings.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                game.setScreen(new MotionSettings(game));
            }
        });

        buttonFormSettings = new TextButton(BUTTON_FORM_TEXT, uiSkin);
        buttonFormSettings.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                game.setScreen(new FormSettings(game));
            }
        });

        buttonScreenSettings = new TextButton(BUTTON_SCREEN_TEXT, uiSkin);
        buttonScreenSettings.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                game.setScreen(new ScreenSettings(game, true));
            }
        });

        buttonStaircaseSettings = new TextButton(BUTTON_STAIR_TEXT, uiSkin);
        buttonStaircaseSettings.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                game.setScreen(new StaircaseSettings(game));
            }
        });

        buttonInputSettings = new TextButton(BUTTON_INPUT_TEXT, uiSkin);

        buttonInputSettings.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                buttonInputSettings.setColor(1, 0, 0, 1);
                game.setScreen(new InputSettings(game));
            }
        });

        buttonReset = new TextButton(BUTTON_RESET_TEXT, uiSkin);
        buttonReset.setColor(Color.LIGHT_GRAY);
        buttonReset.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                dialog = new Dialog("Reset all settings?", uiSkin) {
                    protected void result(Object object) {
                        if (object.equals(true)) {
                            Settings.resetAllSettings();
                        }
                    }
                };
                dialog.text("");
                dialog.getButtonTable().defaults().width(Gdx.graphics.getWidth() * 100f / 1280f);
                TextButton yes = new TextButton("Yes", uiSkin);
                yes.setColor(Color.GREEN);
                dialog.button(yes, true);
                TextButton no = new TextButton("No", uiSkin);
                no.setColor(Color.valueOf("FF3333"));
                dialog.button(no, false);
                dialogIsShown = true;
                dialog.show(stage);
            }
        });

        buttonBack = new TextButton(BUTTON_BACK_TEXT, uiSkin);
        buttonBack.setColor(Color.ORANGE);
        buttonBack.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                game.setScreen(new MainMenu(game));

            }
        });
    }

    /**
     * Populate the table with the UI elements
     */
    private void buildUi() {
        table.setX(viewport.getScreenWidth() / 2);
        table.setY(viewport.getScreenHeight() / 2);

        table.add(buttonMotionSettings).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonFormSettings).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonScreenSettings).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonStaircaseSettings).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonInputSettings).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonReset).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();
        table.add(buttonBack).width(viewport.getScreenWidth() / 3).
                height(viewport.getScreenHeight() / 10).pad(5).row();

        stage.addActor(table);
    }

    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK) || Gdx.input.isKeyJustPressed(Settings.getInputKeyBack())) {
            if (dialogIsShown) {
                dialogIsShown = false;
                dialog.hide();
                game.backPushed = true;
            }
            else if (!game.backPushed) {
                game.backPushed = true;
                game.setScreen(new MainMenu(game));
            }
        }
        else if(game.backPushed) {
            game.backPushed = false;
        }
        stage.act();
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(width / 2);
        table.setY(height / 2);
    }

    @Override
    public void pause() {

    }

    @Override
    public void resume() {

    }

    @Override
    public void hide() {

    }

    @Override
    public void dispose() {
        uiSkin.dispose();
    }

}
