package no.ntnu.master.motion;

import com.badlogic.gdx.math.Rectangle;

import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import no.ntnu.master.objects.Dot;
import no.ntnu.master.utils.Enums.Direction;
import no.ntnu.master.utils.Enums.WorldStates;
import no.ntnu.master.utils.Psychophysics;
import no.ntnu.master.utils.QuadTree;
import no.ntnu.master.utils.Settings;
import no.ntnu.master.utils.World;

/**
 * Created by B on 16.12.2015.
 * Class for holding the World
 */
public class MotionWorld implements World {

    private WorldStates currentState;

    public ArrayList<Dot> dotsLeft;
    public ArrayList<Dot> dotsRight;

    public Rectangle rectLeft;
    public Rectangle rectRight;
    public float patchGap;

    private float leftMinWidth;
    private float leftMaxWidth;

    private float lrMinHeight;
    private float lrMaxHeight;

    private float rightMinWidth;
    private float rightMaxWidth;

    private boolean allRandom;

    private String coherentPatchSide;

    //TODO remember to make this private
    public float coherencePercent;

    private float numberOfDots;
    private float dotSpacing;
    private float dotRadius;
    private float dotMaxAliveTime;

    private float maxRunningTime;
    private float runningTime;

    private SecureRandom random;

    //TODO Remove these before final version
    public int correctAnswerCounter = 0;
    public int wrongAnswerCounter = 0;
    // --Commented out by Inspection (18.02.2016 23:03):public int reversalCounter = 0;

    private QuadTree quadTree;

    private float waitTimer;

    /**
     * Creates a new World for the Motion test.
     */
    public MotionWorld() {
        currentState = WorldStates.PAUSED;
        random = new SecureRandom();
        runningTime = 0f;

        this.dotsLeft = new ArrayList<>();
        this.dotsRight = new ArrayList<>();

        coherencePercent = Settings.getDotCoherency();

        numberOfDots = Settings.getDotMaxAmount();
        dotRadius = Settings.getDotRadiusPixels();
        dotSpacing = (dotRadius*2)+Settings.getDotSpacing();
        maxRunningTime = Settings.getDotAnimationTime();
        dotMaxAliveTime = Settings.getDotMaxLifeTime();

        waitTimer = 0f;

        createPatches();

        calculateMaxMin();

        createDots();
    }

    public void update(float delta) {
//        reversalCounter = InputHandler.getReversalCounter();
        switch (currentState) {
            case FINISHED: break;
            case PAUSED: waitTimer = 0f; paused(); break;
            case RUNNING:
                if ((waitTimer+=delta) > 0.0f) {
                   updateDots(delta);
//                   currentState = WorldStates.PAUSED;
                   break;
               }
       }
    }

    /**
     * Called from update loop when the world is in pause
     * Creates new dots IF current running time is equal to, or larger than maximum running time
     */
    private void paused() {
        if (runningTime >= maxRunningTime) {
            runningTime = 0;
            dotsLeft.clear();
            dotsRight.clear();
        }
    }

    /**
     * Called from update loop when the world is running
     */
    private void updateDots(float delta) {
        //check if the rendering of dots should stop
        runningTime += delta;
        if (runningTime >= maxRunningTime) {
            currentState = WorldStates.PAUSED;
            return;
        }

        quadTree.clear();
        for (int i =0; i < dotsLeft.size();i++) {
            quadTree.insert(dotsLeft.get(i));
        }

        //update dots

        for (int i = 0; i< dotsLeft.size();i++) {
            dotsLeft.get(i).update(delta);
        }

        for (int i = 0; i< dotsLeft.size();i++) {
            if(dotsLeft.get(i).isRandom()) {
                wallCollisionLeftPatch(dotsLeft.get(i), delta);
            }
        }

        List<Dot> returnObjectsL = new ArrayList<>();
        for (int i=0; i<dotsLeft.size();i++) {
            returnObjectsL.clear();
            quadTree.retrieve(returnObjectsL, dotsLeft.get(i));

            for (int j = 0; j< returnObjectsL.size();j++) {
                dotsLeft.get(i).detectDotCollision(returnObjectsL, delta);
            }
        }

        for (int i = 0; i<dotsLeft.size();i++) {
            if (dotsLeft.get(i).changeFlag) {
                dotsLeft.get(i).resetAliveTimer();
            }
        }

        if (dotMaxAliveTime > 0) {
            for (int i = 0; i<dotsLeft.size();i++) {
                if (dotsLeft.get(i).getAliveTimer() <= 0f) {
                    float x = random.nextFloat() * (leftMaxWidth - leftMinWidth) + leftMinWidth;
                    float y = random.nextFloat() * (lrMaxHeight - lrMinHeight) + lrMinHeight;
                    while(!freeSpot(x, y, dotsLeft)) {
                        x = random.nextFloat() * (leftMaxWidth - leftMinWidth) + leftMinWidth;
                        y = random.nextFloat() * (lrMaxHeight - lrMinHeight) + lrMinHeight;
                    }
                    dotsLeft.get(i).setPosition(x, y);
                    //dotsLeft.get(i).resetAliveTimer();
                }
            }
        }

        quadTree.clear();
        for (int i =0; i < dotsRight.size();i++) {
            quadTree.insert(dotsRight.get(i));
        }

        for (int i = 0; i< dotsRight.size();i++) {
            dotsRight.get(i).update(delta);
        }
        for (int i = 0; i< dotsRight.size();i++) {
            if(dotsRight.get(i).isRandom()) {
                wallCollisionRightPatch(dotsRight.get(i), delta);
            }
        }

        List<Dot> returnObjectsR = new ArrayList<>();
        for (int i=0; i<dotsRight.size();i++) {
            returnObjectsR.clear();
            quadTree.retrieve(returnObjectsR, dotsRight.get(i));

            for (int j = 0; j< returnObjectsR.size();j++) {
                dotsRight.get(i).detectDotCollision(returnObjectsR, delta);
            }
        }

        for (int i = 0; i<dotsRight.size();i++) {
            if (dotsRight.get(i).changeFlag) {
                dotsRight.get(i).resetAliveTimer();
            }
        }

        if (dotMaxAliveTime > 0) {
            for (int i = 0; i<dotsRight.size();i++) {
                if (dotsRight.get(i).getAliveTimer() <= 0f) {
                    float x = random.nextFloat() * (rightMaxWidth - rightMinWidth) + rightMinWidth;
                    float y = random.nextFloat() * (lrMaxHeight - lrMinHeight) + lrMinHeight;
                    while(!freeSpot(x, y, dotsRight)) {
                        x = random.nextFloat() * (rightMaxWidth - rightMinWidth) + rightMinWidth;
                        y = random.nextFloat() * (lrMaxHeight - lrMinHeight) + lrMinHeight;
                    }
                    dotsRight.get(i).setPosition(x, y);
                }
            }
        }
    }

    private void createPatches() {

        patchGap = Psychophysics.getPatchSeparationInPixels();
        float patchWidth = Psychophysics.getPatchWidthInPixels();
        float patchHeight = Psychophysics.getPatchHeightInPixels();

        float screenXCenter = Settings.getScreenWidthPixels() / 2;
        float screenYCenter = Settings.getScreenHeightPixels() / 2;

        rectLeft = new Rectangle(screenXCenter-patchWidth-(patchGap /2), screenYCenter-(patchHeight/2),
                patchWidth, patchHeight);
        rectRight = new Rectangle(rectLeft.x+patchWidth+ patchGap, screenYCenter-(patchHeight/2),
                patchWidth ,patchHeight);

        quadTree = new QuadTree(0,new Rectangle(rectLeft.x, rectLeft.y,
                rectLeft.width+patchGap+rectRight.width, rectLeft.height));
    }

    private void calculateMaxMin() {
        leftMinWidth = rectLeft.x + dotRadius;
        leftMaxWidth = (rectLeft.x+rectLeft.width) - dotRadius - dotRadius;

        lrMinHeight = rectRight.y + dotRadius;
        lrMaxHeight = (rectRight.y + rectRight.height) - dotRadius - dotRadius;

        rightMinWidth = rectRight.x + dotRadius;
        rightMaxWidth = (rectRight.x+rectRight.width) - dotRadius - dotRadius;
    }

    /**
     * Fill the left and right patch with dots at random locations. A portion of the dots in a
     * random patch will be set to move coherently either leftwards or rightwards.
     * Needs optimizing -.-
     */
    private void createDots() {
        float x;
        float y;
        int numberOfCoherent = 0;
        int killingPercentage = Settings.getDotPercentageToKill();
        float test = ((numberOfDots * killingPercentage)/100);
        int times = 1;

        //Choose direction of coherent moving dots, 0 is left, 1 is right
        Direction coherentDirection = Direction.get(random.nextInt(2));

        //choose patch to contain random dots
        allRandom = random.nextBoolean();
        if (!allRandom) {
            coherentPatchSide = "Left";
        } else coherentPatchSide = "Right";

        for (int i = 0;i < numberOfDots;i++) {
            if (i == test*times) {
                times++;
            }

            //create dots for the left patch
            do {
                x = random.nextFloat() * (leftMaxWidth - leftMinWidth) + leftMinWidth;
                y = random.nextFloat() * (lrMaxHeight - lrMinHeight) + lrMinHeight;
            } while (!freeSpot(x, y, dotsLeft));
            if (coherentPatchSide.equals("Left")
                    && (numberOfCoherent / numberOfDots) * 100f < coherencePercent) {
                //create X amount of coherent moving dots
                dotsLeft.add(new Dot(x, y, dotRadius, coherentDirection, (dotMaxAliveTime*times)));
                numberOfCoherent++;
            } else {
                //Fill up with random dots
                dotsLeft.add(new Dot(x, y, dotRadius, Direction.RANDOM, (dotMaxAliveTime * times)));
            }

            //Create dots for the right patch
            do {
                x = random.nextFloat() * (rightMaxWidth - rightMinWidth) + rightMinWidth;
                y = random.nextFloat() * (lrMaxHeight - lrMinHeight) + lrMinHeight;
            } while (!freeSpot(x, y, dotsRight));
            if (coherentPatchSide.equals("Right")
                    && (numberOfCoherent / numberOfDots) * 100f < coherencePercent) {
                //create X amount of coherent moving dots
                dotsRight.add(new Dot(x, y, dotRadius, coherentDirection, (dotMaxAliveTime*times)));
                numberOfCoherent++;
            } else {
                //Fill up with random dots
                dotsRight.add(new Dot(x, y, dotRadius, Direction.RANDOM, (dotMaxAliveTime * times)));
            }

        }
    }


    /**
     * Checks if there already is a dot at location
     * @param x location of dot
     * @param y location of dot
     * @param dots the array containing dots to compare with
     * @return true if spot is free
     */
    private boolean freeSpot(float x, float y, ArrayList<Dot> dots) {
        for (Iterator<Dot> it = dots.iterator();it.hasNext();) {
            Dot dot = it.next();
            if (dot.getPosition().dst(x, y) < dotSpacing) {
                return false;
            }
        }
        return true;
    }

    private void wallCollisionLeftPatch(Dot dot, float delta) {
        if (dot.getPosition().x <= rectLeft.x || dot.getPosition().x >= leftMaxWidth) {
            dot.wallCollision(true, delta);
        }
        if (dot.getPosition().y <= rectLeft.y || dot.getPosition().y >= lrMaxHeight) {
            dot.wallCollision(false, delta);
        }
    }

    private void wallCollisionRightPatch(Dot dot, float delta) {
        if (dot.getPosition().x <= rectRight.x || dot.getPosition().x >= rightMaxWidth) {
            dot.wallCollision(true, delta);
        }
        if (dot.getPosition().y <= rectRight.y || dot.getPosition().y >= lrMaxHeight) {
            dot.wallCollision(false, delta);
        }
    }

    @Override
    public WorldStates getState() {
        return this.currentState;
    }

    @Override
    public void setState(WorldStates worldState) {
        this.currentState = worldState;
    }

    @Override
    public String getCoherentPatch() {
        return coherentPatchSide;
    }

    @Override
    public float getCoherencyPercent() { return coherencePercent; }

    @Override
    public boolean getIsFixed() { return true; }

    @Override
    public void updateCoherency(float factor, boolean isCorrectAnswer) {

        float temp = coherencePercent * factor;

        //Update coherency and counters
        if (isCorrectAnswer) {
            if (factor > 1) {
                temp -= coherencePercent;
                coherencePercent -= temp;
            }
            else {
                coherencePercent = temp;
            }
            correctAnswerCounter++;
        }
        else {
            if (factor > 1) {
                coherencePercent = temp;
            }
            else {
                temp -= coherencePercent;
                coherencePercent -= temp;
            }
            if (coherencePercent > 100) {
                coherencePercent = 100;
            }
            wrongAnswerCounter++;
        }

        //Create new dots
        dotsLeft.clear();
        dotsRight.clear();
        createDots();
        waitTimer = 0;
        runningTime = 0;
    }
}
