package no.ntnu.master.settingcreens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.SettingsMenu;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.CheckBoxReversed;
import no.ntnu.master.utils.Psychophysics;
import no.ntnu.master.utils.Settings;

/**
 * Created by B on 06.01.2016.
 * Settings screen related to how the Form tests should behave
 */
public class FormSettings extends ScreenAdapter {

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;

    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;

    /**
     * Variables used to create the user interface
     */
    private Label screenHeader;
    private Label amountLabel;
    private Label diameterWbLabel;
    private Label circlesLabel;
    private Label circlesGapLabel;
    private Label lineLengthLabel;
    private Label lineHeightLabel;
    private Label lineHeightPxLabel;
    private Label lineGapLabel;
    private Label coherencyLabel;
    private Label fDetectionTimeLabel;
    private Label rDetectionTimeLabel;

    private CheckBoxReversed autoCheckBox;
    private TextField amountField;
    private TextField diameterWbField;
    private TextField circlesField;
    private TextField circlesGapField;
    private TextField lineLengthField;
    private TextField lineHeightField;
    private TextField lineGapField;
    private TextField coherencyField;
    private TextField fDetectionTimeField;
    private TextField rDetectionTimeField;

    private TextButton buttonCancel, buttonDone;


    /**
     * Label text
     */
    private final String AUTO_CHECKBOX_TEXT = "Auto mode";
    private final String SCREEN_HEADER_TEXT = "Form Settings";
    private final String AMOUNT_TEXT = "Number of lines";
    private final String DIAMETER_WB_TEXT = "Diameter"+ (char) 0x00B0;
    private final String CIRCLES_TEXT = "Nr of circles";
    private final String CIRCLES_GAP_TEXT = "Circle gap"+ (char) 0x00B0;
    private final String LINE_LENGTH_TEXT = "Line length"+ (char) 0x00B0;
    private final String LINE_HEIGHT_TEXT = "Line height px";
    private final String LINE_GAP_TEXT = "Line gap"+ (char) 0x00B0;
    private final String COHERENCY_TEXT = "Initial coherency";
    private final String F_DETECTION_TIME_TEXT = "Fixed\ndetection time";
    private final String R_DETECTION_TIME_TEXT = "Random\ndetection time";


    /**
     * Button text
     */
    private final String BUTTON_DONE_TEXT = "Done";
    private final String BUTTON_CANCEL_TEXT = "Cancel";

    /**
     * A stage and table for holding the various user interface elements
     */
    private Table table;
    private Stage stage;
    private ScrollPane scrollPane;

    /**
     * Reference to the game object
     * Used for changing screens
     */
    private DyslexiaDetection game;

    /**
     * Decimal format
     */
    private DecimalFormatSymbols dfs;
    private DecimalFormat df;

    /**
     * Constructor for FormSettings
     * @param game the game object reference
     */
    public FormSettings(DyslexiaDetection game) {
        this.game = game;

        dfs = new DecimalFormatSymbols();
        dfs.setDecimalSeparator('.');
        df = new DecimalFormat("0.0000", dfs);

        initScreen();
        initUiElements();
        buildUi();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initScreen() {
        //setup a camera to show the screen
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();
        scrollPane = new ScrollPane(table, uiSkin);

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create and fill the stage with the gui elements
     */
    private void initUiElements() {

        screenHeader = new Label(SCREEN_HEADER_TEXT, uiSkin);

        amountLabel = new Label(AMOUNT_TEXT, uiSkin);
        amountField = new TextField(""+Settings.getFormMaxLineSegments(), uiSkin);
        amountField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]"," +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });

        diameterWbLabel = new Label(DIAMETER_WB_TEXT, uiSkin);
        diameterWbField = new TextField(""+Settings.getFormDiameterWb(), uiSkin);
        diameterWbField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        diameterWbField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    float d = Psychophysics.visualAngleInPixels(
                            Float.parseFloat(textField.getText()),
                            Settings.getScreenViewingDistance(),
                            Settings.getScreenWidthPixels(),
                            Settings.getScreenWidth()
                    );
                    float temp = Settings.getFormCirclesGap();
                    if (circlesGapField.getText().length() > 0) {
                        temp = Float.parseFloat(circlesGapField.getText());
                    }
                    float circleGap = Psychophysics.visualAngleInPixels(
                            temp,
                            Settings.getScreenViewingDistance(),
                            Settings.getScreenWidthPixels(),
                            Settings.getScreenWidth()
                    );
                    float value = Math.round((d / circleGap) / 2);

                    circlesField.setText("" + (int) value);
                }
            }
        });

        circlesLabel = new Label(CIRCLES_TEXT, uiSkin);
        circlesField = new TextField(""+Settings.getFormNrOfCircles(), uiSkin);
        circlesField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });

        circlesGapLabel = new Label(CIRCLES_GAP_TEXT, uiSkin);
        circlesGapField = new TextField(""+Settings.getFormCirclesGap(), uiSkin);
        circlesGapField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        circlesGapField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    float temp = Settings.getFormDiameterWb();
                    if (diameterWbField.getText().length() > 0) {
                        temp = Float.parseFloat(diameterWbField.getText());
                    }
                    float d = Psychophysics.visualAngleInPixels(
                            temp,
                            Settings.getScreenViewingDistance(),
                            Settings.getScreenWidthPixels(),
                            Settings.getScreenWidth()
                    );
                    float circleGap = Psychophysics.visualAngleInPixels(
                            Float.parseFloat(textField.getText()),
                            Settings.getScreenViewingDistance(),
                            Settings.getScreenWidthPixels(),
                            Settings.getScreenWidth()
                    );
                    float value = Math.round((d / circleGap) / 2);
                    circlesField.setText("" + (int) value);
                }
            }
        });

        lineLengthLabel = new Label(LINE_LENGTH_TEXT, uiSkin);
        lineLengthField = new TextField(""+Settings.getFormLineLength(), uiSkin);
        lineLengthField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        lineHeightLabel = new Label(LINE_HEIGHT_TEXT, uiSkin);
        lineHeightField = new TextField(""+Settings.getFormLineHeight(), uiSkin);
        lineHeightField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        lineHeightField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    float temp = Psychophysics.visualAngleInDegrees(
                            Float.parseFloat(textField.getText()),
                            Settings.getScreenViewingDistance(),
                            Settings.getScreenHeightPixels(),
                            Settings.getScreenHeight()

                    );
                    lineHeightPxLabel.setText("~"+df.format(temp)+ (char)0x00B0);
                }
            }
        });

        float temp = Psychophysics.visualAngleInDegrees(
                Settings.getFormLineHeight(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenHeightPixels(),
                Settings.getScreenHeight()

        );
        lineHeightPxLabel = new Label("~"+df.format(temp)+ (char)0x00B0,uiSkin);

        lineGapLabel = new Label(LINE_GAP_TEXT, uiSkin);
        lineGapField = new TextField(""+Settings.getFormLineGap(), uiSkin);
        lineGapField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        coherencyLabel = new Label(COHERENCY_TEXT, uiSkin);
        coherencyField = new TextField(""+Settings.getFormCoherency(), uiSkin);
        coherencyField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        coherencyField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    if (Float.parseFloat(textField.getText()) > 100f) {
                        textField.setText(String.valueOf(100f));
                    }
                }
            }
        });

        fDetectionTimeLabel = new Label(F_DETECTION_TIME_TEXT, uiSkin);
        fDetectionTimeField = new TextField(""+Settings.getFormFDetectionTime(), uiSkin);
        fDetectionTimeField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]"," +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        rDetectionTimeLabel = new Label(R_DETECTION_TIME_TEXT, uiSkin);
        rDetectionTimeField = new TextField(""+Settings.getFormRDetectionTime(), uiSkin);
        rDetectionTimeField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        buttonDone = new TextButton(BUTTON_DONE_TEXT, uiSkin);
        buttonDone.setColor(Color.GREEN);
        buttonDone.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                storeContent();
                Gdx.input.setOnscreenKeyboardVisible(false);
                game.setScreen(new SettingsMenu(game));
            }
        });

        buttonCancel = new TextButton(BUTTON_CANCEL_TEXT, uiSkin);
        buttonCancel.setColor(Color.ORANGE);
        buttonCancel.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.input.setOnscreenKeyboardVisible(false);
                game.setScreen(new SettingsMenu(game));
            }
        });

        autoCheckBox = new CheckBoxReversed(AUTO_CHECKBOX_TEXT, uiSkin);
        autoCheckBox.setChecked(Settings.getFormAutoMode());
        autoCheckBox.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                circlesField.setDisabled(!circlesField.isDisabled());
                circlesGapField.setDisabled(!circlesGapField.isDisabled());
                lineGapField.setDisabled(!lineGapField.isDisabled());
            }
        });
        if(autoCheckBox.isChecked()) {
            //disable un-relevant fields
            circlesField.setDisabled(true);
            circlesGapField.setDisabled(true);
            lineGapField.setDisabled(true);
        }
    }

    private void buildUi() {

        float prefW = (float)Math.ceil(Gdx.graphics.getWidth() * 140 / 1280);

        scrollPane.setFillParent(true);

        //Simple fix for onScreenKeyboard covering input fields
        table.padBottom(cam.viewportHeight/2);

        table.add(screenHeader).colspan(2).center().pad(20).row();

        table.add(autoCheckBox).colspan(2).center().pad(20).row();

        table.add(amountLabel).left();
        table.add(diameterWbLabel).right().row();

        table.add(amountField).prefWidth(prefW).left().padBottom(5);
        table.add(diameterWbField).prefWidth(prefW).right().padBottom(5).row();

        table.add(circlesLabel).left();
        table.add(circlesGapLabel).right().row();

        table.add(circlesField).prefWidth(prefW).left().padBottom(5);
        table.add(circlesGapField).prefWidth(prefW).right().padBottom(5).row();

        table.add(lineLengthLabel).left();
        table.add(lineHeightLabel).right().row();

        table.add(lineLengthField).prefWidth(prefW).left().padBottom(5);
        table.add(lineHeightField).prefWidth(prefW).right().padBottom(5);
        table.add(lineHeightPxLabel).left().padBottom(5).width(10).row();

        table.add(lineGapLabel).left().row();
        table.add(lineGapField).prefWidth(prefW).left().padBottom(5).row();

        table.add(coherencyLabel).left().padRight(40f).row();
        table.add(coherencyField).prefWidth(prefW).left().padBottom(5).row();

        table.add(fDetectionTimeLabel).left();
        table.add(rDetectionTimeLabel).right().row();

        table.add(fDetectionTimeField).prefWidth(prefW).left().padBottom(5);
        table.add(rDetectionTimeField).prefWidth(prefW).right().padBottom(5).row();

        table.add(buttonDone).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5).padBottom(5).row();

        table.add(buttonCancel).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5).padBottom(5).row();

        stage.addActor(scrollPane);
    }

    private void storeContent() {
        try {

            Settings.setFormAutoMode(autoCheckBox.isChecked());

            Settings.setFormMaxLineSegments(Integer.parseInt(amountField.getText()));

            Settings.setFormDiameterWb(Float.parseFloat(diameterWbField.getText()));

            Settings.setFormNrOfCircles(Integer.parseInt(circlesField.getText()));

            Settings.setFormCirclesGap(Float.parseFloat(circlesGapField.getText()));

            Settings.setFormLineLength(Float.parseFloat(lineLengthField.getText()));

            Settings.setFormLineHeight(Float.parseFloat(lineHeightField.getText()));

            Settings.setFormLineGap(Float.parseFloat(lineGapField.getText()));

            Settings.setFormCoherency(Float.parseFloat(coherencyField.getText()));

            Settings.setFormFDetectionTime(Float.parseFloat(fDetectionTimeField.getText()));

            Settings.setFormRDetectionTime(Float.parseFloat(rDetectionTimeField.getText()));

        } catch (NumberFormatException exp) {
            Gdx.app.log("FORM SETTINGS", "Exception in storing content from fields " + exp);
        }
    }

    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK) || Gdx.input.isKeyJustPressed(Settings.getInputKeyBack())) {
            game.backPushed = true;
            game.setScreen(new SettingsMenu(game));
        }
        stage.act();
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(width/2);
        table.setY(height/2);
        //stage.getViewport().update(width, height);
    }

    @Override
    public void pause() {
        storeContent();
    }

    @Override
    public void dispose() {
        storeContent();
        uiSkin.dispose();
    }
}
