package no.ntnu.master.settingcreens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Dialog;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.SettingsMenu;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.CheckBoxReversed;
import no.ntnu.master.utils.Enums;
import no.ntnu.master.utils.Settings;

/**
 * Created by B on 15.02.2016.
 * Option screen for input parameters
 */
public class InputSettings extends ScreenAdapter {

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;

    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;

    /**
     * Variables used to create the user interface
     */
    private Label screenHeader;
    private Label keyLeftLabel;
    private Label keyRightLabel;
    private Label keyBackLabel;
    private TextField keyLeftField;
    private TextField keyRightField;
    private TextField keyBackField;
    private CheckBoxReversed continuousModeCheck;
    private TextButton buttonDone, buttonCancel;

    private Dialog inputDialog;
    private Enums.Direction calledFrom;

    /**
     * Label text
     */
    private final String SCREEN_HEADER_TEXT = "Input Settings";
    private final String KEY_LEFT_TEXT = "Choose left patch key: ";
    private final String KEY_RIGHT_TEXT = "Choose right patch key: ";
    private final String KEY_BACK_TEXT = "Back key: ";
    private final String CONTINUOUS_MODE_TEXT = "Continuous mode:";

    /**
     * Button text
     */
    private final String BUTTON_DONE_TEXT = "Done";
    private final String BUTTON_CANCEL_TEXT = "Cancel";

    /**
     * A stage and table for holding the various user interface elements
     */
    private Table table;
    private Stage stage;

    /**
     * Reference to the game object
     * Used for changing screens
     */
    private DyslexiaDetection game;

    public InputSettings(DyslexiaDetection game) {
        this.game = game;
        initScreen();
        initUiElements();
        buildUi();
    }

    private void initScreen() {
        //setup a camera to show the screen
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    private void initUiElements() {

        inputDialog = new Dialog("Press key to use", uiSkin);
        inputDialog.text("");
        inputDialog.setVisible(false);
        inputDialog.addListener(new InputListener() {
            public boolean keyUp(InputEvent event, int keycode) {
                switch (calledFrom) {
                    case LEFT:
                        keyLeftField.setText(Input.Keys.toString(keycode));
                        keyLeftField.setDisabled(true);
                        break;
                    case RIGHT:
                        keyRightField.setText(Input.Keys.toString(keycode));
                        keyRightField.setDisabled(true);
                        break;
                    case RANDOM:
                        keyBackField.setText(Input.Keys.toString(keycode));
                        keyBackField.setDisabled(true);
                        break;
                    default:
                        break;
                }
                inputDialog.hide();
                inputDialog.setVisible(false);
                return false;
            }
        });

        screenHeader = new Label(SCREEN_HEADER_TEXT, uiSkin);

        keyLeftLabel = new Label(KEY_LEFT_TEXT, uiSkin);
        keyLeftField = new TextField(""+(Input.Keys.toString(Settings.getInputKeyLeft())), uiSkin);
        keyLeftField.addListener(new InputListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                calledFrom = Enums.Direction.LEFT;
                inputDialog.setVisible(true);
                inputDialog.show(stage);
                return false;
            }
            @Override
            public boolean keyDown(InputEvent event, int keycode) {
                calledFrom = Enums.Direction.LEFT;
                inputDialog.show(stage);
                return false;
            }
        });

        keyRightLabel = new Label(KEY_RIGHT_TEXT, uiSkin);
        keyRightField = new TextField(""+(Input.Keys.toString(Settings.getInputKeyRight())), uiSkin);
        keyRightField.addListener(new InputListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                calledFrom = Enums.Direction.RIGHT;
                inputDialog.setVisible(true);
                inputDialog.show(stage);
                return false;
            }
            @Override
            public boolean keyDown(InputEvent event, int keycode) {
                calledFrom = Enums.Direction.RIGHT;
                inputDialog.show(stage);
                return false;
            }
        });

        keyBackLabel = new Label(KEY_BACK_TEXT, uiSkin);
        keyBackField = new TextField(""+(Input.Keys.toString(Settings.getInputKeyBack())), uiSkin);
        keyBackField.addListener(new InputListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                calledFrom = Enums.Direction.RANDOM;
                inputDialog.setVisible(true);
                inputDialog.show(stage);
                return false;
            }
            @Override
            public boolean keyDown(InputEvent event, int keycode) {
                calledFrom = Enums.Direction.RANDOM;
                inputDialog.show(stage);
                return false;
            }
        });

        continuousModeCheck = new CheckBoxReversed(CONTINUOUS_MODE_TEXT, uiSkin);
        continuousModeCheck.setChecked(Settings.getInputContinuousMode());

        buttonDone = new TextButton(BUTTON_DONE_TEXT, uiSkin);
        buttonDone.setColor(Color.GREEN);
        buttonDone.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.input.setOnscreenKeyboardVisible(false);
                storeContent();
                game.setScreen(new SettingsMenu(game));
            }
        });

        buttonCancel = new TextButton(BUTTON_CANCEL_TEXT, uiSkin);
        buttonCancel.setColor(Color.ORANGE);
        buttonCancel.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.input.setOnscreenKeyboardVisible(false);
                game.setScreen(new SettingsMenu(game));
            }
        });

    }

    private void buildUi() {

        float prefW = (float)Math.ceil(Gdx.graphics.getWidth() * 140 / 1280);

        //Simple fix for onScreenKeyboard covering input fields

        table.add(screenHeader).colspan(2).center().pad(20).row();

//        if (Gdx.app.getType().equals(Application.ApplicationType.Desktop)) {
            table.add(keyLeftLabel).left();
            table.add(keyLeftField).prefWidth(prefW).padBottom(5).row();

            table.add(keyRightLabel).left();
            table.add(keyRightField).prefWidth(prefW).padBottom(5).row();

            table.add(keyBackLabel).left();
            table.add(keyBackField).prefWidth(prefW).padBottom(5).row();
//        }

        table.add(continuousModeCheck).colspan(2).left().padBottom(20).row();

        table.add(buttonDone).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5f).padBottom(5f).row();

        table.add(buttonCancel).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5f).padBottom(5f).row();

        stage.addActor(table);
    }

    private void storeContent() {
        try {

            Settings.setInputKeyLeft(Input.Keys.valueOf(keyLeftField.getText()));

            Settings.setInputKeyRight(Input.Keys.valueOf(keyRightField.getText()));

            Settings.setInputKeyBack(Input.Keys.valueOf(keyBackField.getText()));

            Settings.setInputContinuousMode(continuousModeCheck.isChecked());

        } catch (NumberFormatException exp) {
            Gdx.app.log("INPUT_OPTIONS", "Exception in storing content from fields " + exp);
        }
    }

    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK) ||
                (Gdx.input.isKeyJustPressed(Settings.getInputKeyBack()) && !inputDialog.isVisible())) {
            game.backPushed = true;
            game.setScreen(new SettingsMenu(game));
        }
        stage.act();
        keyLeftField.setDisabled(false);
        keyRightField.setDisabled(false);
        keyBackField.setDisabled(false);
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(width / 2);
        table.setY(height / 2);
        //stage.getViewport().update(width, height);
    }

    @Override
    public void pause() {
        storeContent();
    }

    @Override
    public void dispose() {
        storeContent();
        uiSkin.dispose();
    }
}
