package no.ntnu.master.settingcreens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.SettingsMenu;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.Psychophysics;
import no.ntnu.master.utils.Settings;

/**
 * Created by B on 06.01.2016.
 * A settings screen for Motion specific stuff
 */
public class MotionSettings extends ScreenAdapter {

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;

    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;

    /**
     * Variables used to create the user interface
     */
    private Label screenHeader;
    private Label numberOfDotsLabel;
    private Label radiusPixelsLabel;
    private Label dotSpacingLabel;
    private Label velocityLabel;
    private Label coherencyLabel;
    private Label animationTimeLabel;
    private Label maxLifeTimeLabel;
    private Label toKillLabel;
    private Label horReversalTimeLabel;
    private Label randDirTimeLabel;

    private TextField numberOfDotsField;
    private TextField radiusPixelsField;
    private TextField dotSpacingField;
    private TextField velocityField;
    private TextField coherencyField;
    private TextField animationTimeField;
    private TextField maxLifeTimeField;
    private TextField toKillField;
    private TextField horReversalTimeField;
    private TextField randDirTimeField;

    private TextButton buttonDone, buttonCancel;

    /**
     * Label text
     */
    private final String SCREEN_HEADER_TEXT = "Motion Settings";
    private final String NUMBER_OF_DOTS_TEXT = "Number of dots";
    private final String RADIUS_PIXELS_TEXT = "Radius in pixels";
    private final String DOT_SPACING_TEXT = "Initial spacing";
    private final String VELOCITY_TEXT = "Velocity";
    private final String COHERENCY_LABEL_TEXT = "Coherency start %";
    private final String ANIMATION_LABEL_TEXT = "Animation time";
    private final String MAX_LIFE_LABEL_TEXT = "Dot alive time";
    private final String TO_KILL_LABEL_TEXT = "Percentage to destroy";
    private final String REVERSAL_LABEL_TEXT = "Coherent reversal";
    private final String RANDOM_LABEL_TEXT = "Random reversal";

    /**
     * Button text
     */
    private final String BUTTON_DONE_TEXT = "Done";
    private final String BUTTON_CANCEL_TEXT = "Cancel";

    /**
     * A stage and table for holding the various user interface elements
     */
    private Table table;
    private Stage stage;
    private ScrollPane scrollPane;

    /**
     * Reference to the game object
     * Used for changing screens
     */
    private DyslexiaDetection game;

    public MotionSettings(DyslexiaDetection game) {
        this.game = game;
        initScreen();
        initUiElements();
        buildUi();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initScreen() {
        //setup a camera to show the screen
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();
        scrollPane = new ScrollPane(table, uiSkin);

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create and fill the stage with the gui elements
     */
    private void initUiElements() {

        screenHeader = new Label(SCREEN_HEADER_TEXT, uiSkin);

        numberOfDotsLabel = new Label(NUMBER_OF_DOTS_TEXT, uiSkin);
        numberOfDotsField = new TextField(""+Settings.getDotMaxAmount(), uiSkin);
        numberOfDotsField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]"," +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        numberOfDotsField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                int patchWidth = (int) Psychophysics.getPatchWidthInPixels();
                int patchHeight = (int) Psychophysics.getPatchHeightInPixels();
                float d = Settings.getDotRadiusPixels() * 2;
                float s = d + Settings.getDotSpacing();
                int w = (int) (patchWidth / (d + s));
                int h = (int) (patchHeight / (d + s));
                int max = w * h;
                if (textField.getText().length() > 0) {
                    if (Integer.parseInt(textField.getText()) > max) {
                        textField.setText(String.valueOf(max));
                    }
                }
            }
        });

        radiusPixelsLabel = new Label(RADIUS_PIXELS_TEXT, uiSkin);
        radiusPixelsField = new TextField(""+Settings.getDotRadiusPixels(), uiSkin);
        radiusPixelsField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        radiusPixelsField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int patchWidth = (int) Psychophysics.getPatchWidthInPixels();
                    int patchHeight = (int) Psychophysics.getPatchHeightInPixels();
                    float d = Float.parseFloat(textField.getText()) * 2;
                    float s = d + Settings.getDotSpacing();
                    int w = (int) (patchWidth / (d + s));
                    int h = (int) (patchHeight / (d + s));
                    int max = w * h;
                    Settings.setDotRadiusPixels(Float.parseFloat(radiusPixelsField.getText()));
                    if (numberOfDotsField.getText().length() > 0) {
                        if (Integer.parseInt(numberOfDotsField.getText()) > max) {
                            numberOfDotsField.setText(String.valueOf(max));
                        }
                    }
                }
            }
        });

        dotSpacingLabel = new Label(DOT_SPACING_TEXT, uiSkin);
        dotSpacingField = new TextField(""+Settings.getDotSpacing(), uiSkin);
        dotSpacingField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        dotSpacingField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int patchWidth = (int) Psychophysics.getPatchWidthInPixels();
                    int patchHeight = (int) Psychophysics.getPatchHeightInPixels();
                    float d = Settings.getDotRadiusPixels() * 2;
                    float s = d + Float.parseFloat(textField.getText());
                    int w = (int) (patchWidth / (d + s));
                    int h = (int) (patchHeight / (d + s));
                    int max = w * h;
                    Settings.setDotSpacing(Float.parseFloat(textField.getText()));
                    if (numberOfDotsField.getText().length() > 0) {
                        if (Integer.parseInt(numberOfDotsField.getText()) > max) {
                            numberOfDotsField.setText(String.valueOf(max));
                        }
                    }
                }
            }
        });

        velocityLabel = new Label(VELOCITY_TEXT, uiSkin);
        velocityField = new TextField(""+Settings.getDotVelocity(), uiSkin);
        velocityField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]"," +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        coherencyLabel = new Label(COHERENCY_LABEL_TEXT, uiSkin);
        coherencyField = new TextField(""+Settings.getDotCoherency(), uiSkin);
        coherencyField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]", " +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });
        coherencyField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    if (Float.parseFloat(textField.getText()) > 100f) {
                        textField.setText(String.valueOf(100f));
                    }
                }
            }
        });

        animationTimeLabel = new Label(ANIMATION_LABEL_TEXT, uiSkin);
        animationTimeField = new TextField(""+ Settings.getDotAnimationTime(), uiSkin);
        animationTimeField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]"," +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        maxLifeTimeLabel = new Label(MAX_LIFE_LABEL_TEXT, uiSkin);
        maxLifeTimeField = new TextField(""+ Settings.getDotMaxLifeTime(), uiSkin);
        maxLifeTimeField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]"," +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        toKillLabel = new Label(TO_KILL_LABEL_TEXT, uiSkin);
        toKillField = new TextField(""+Settings.getDotPercentageToKill(), uiSkin);
        toKillField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });

        horReversalTimeLabel = new Label(REVERSAL_LABEL_TEXT, uiSkin);
        horReversalTimeField = new TextField(""+Settings.getDotHorizontalReversalTime(), uiSkin);
        horReversalTimeField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]"," +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        randDirTimeLabel = new Label(RANDOM_LABEL_TEXT, uiSkin);
        randDirTimeField = new TextField(""+Settings.getDotRandomDirectionTime(), uiSkin);
        randDirTimeField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.]"," +"));
                Matcher matcher = pattern.matcher("0123456789.");
                return matcher.find();
            }

        });

        buttonDone = new TextButton(BUTTON_DONE_TEXT, uiSkin);
        buttonDone.setColor(Color.GREEN);
        buttonDone.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                storeContent();
                Gdx.input.setOnscreenKeyboardVisible(false);
                game.setScreen(new SettingsMenu(game));
            }
        });

        buttonCancel = new TextButton(BUTTON_CANCEL_TEXT, uiSkin);
        buttonCancel.setColor(Color.ORANGE);
        buttonCancel.addListener(new ClickListener() {
            public void clicked(InputEvent e, float x, float y) {
                Gdx.input.setOnscreenKeyboardVisible(false);
                game.setScreen(new SettingsMenu(game));
            }
        });
    }

    /**
     * Populate the table with ui elements
     */
    private void buildUi() {

        float prefW = (float)Math.ceil(Gdx.graphics.getWidth() * 140 / 1280);
        scrollPane.setFillParent(true);

        //Simple fix for onScreenKeyboard covering input fields
        table.padBottom(cam.viewportHeight / 2);

        table.add(screenHeader).colspan(2).center().pad(20).row();

        table.add(numberOfDotsLabel).left();
        table.add(radiusPixelsLabel).right().row();

        table.add(numberOfDotsField).prefWidth(prefW).left().width(150).padBottom(5);
        table.add(radiusPixelsField).prefWidth(prefW).right().width(150).padBottom(5).row();

        table.add(dotSpacingLabel).left();
        table.add(velocityLabel).right().row();

        table.add(dotSpacingField).prefWidth(prefW).left().width(150).padBottom(5);
        table.add(velocityField).prefWidth(prefW).right().width(150).padBottom(5).row();

        table.add(animationTimeLabel).left();
        table.add(coherencyLabel).right().row();

        table.add(animationTimeField).prefWidth(prefW).left().width(150).padBottom(5);
        table.add(coherencyField).prefWidth(prefW).right().width(150).padBottom(5).row();

        table.add(maxLifeTimeLabel).left();
        table.add(toKillLabel).right().row();

        table.add(maxLifeTimeField).prefWidth(prefW).left().width(150).padBottom(5);
        table.add(toKillField).prefWidth(prefW).right().width(150).padBottom(5).row();

        table.add(horReversalTimeLabel).left();
        table.add(randDirTimeLabel).right().row();

        table.add(horReversalTimeField).prefWidth(prefW).left().width(150).padBottom(5);
        table.add(randDirTimeField).prefWidth(prefW).right().width(150).padBottom(5).row();

        table.add(buttonDone).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5).padBottom(5).row();

        table.add(buttonCancel).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5).padBottom(5).row();

        stage.addActor(scrollPane);
    }

    private void storeContent() {
        try {
            Settings.setDotMaxAmount(Integer.parseInt(numberOfDotsField.getText()));

            Settings.setDotSpacing(Float.parseFloat(dotSpacingField.getText()));

            Settings.setDotRadiusPixels(Float.parseFloat(radiusPixelsField.getText()));

            Settings.setDotVelocity(Float.parseFloat(velocityField.getText()));

            Settings.setDotCoherency(Float.parseFloat(coherencyField.getText()));

            Settings.setDotAnimationTime(Float.parseFloat(animationTimeField.getText()));

            Settings.setDotMaxLifeTime(Float.parseFloat(maxLifeTimeField.getText()));

            Settings.setDotPercentageToKill(Integer.parseInt(toKillField.getText()));

            Settings.setDotHorizontalReversalTime(Float.parseFloat(horReversalTimeField.getText()));

            Settings.setDotRandomDirectionTime(Float.parseFloat(randDirTimeField.getText()));

        } catch (NumberFormatException exp) {
            Gdx.app.log("M SETTINGS", "Exception in storing content from fields " + exp);
        }
    }

    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK) || Gdx.input.isKeyJustPressed(Settings.getInputKeyBack())) {
            game.backPushed = true;
            game.setScreen(new SettingsMenu(game));
        }
        stage.act();
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(width / 2);
        table.setY(height / 2);
        //stage.getViewport().update(width, height);
    }

    @Override
    public void pause() {
        storeContent();
    }

    @Override
    public void dispose() {
        storeContent();
        uiSkin.dispose();
    }
}
