package no.ntnu.master.settingcreens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.MainMenu;
import no.ntnu.master.menus.SettingsMenu;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.Psychophysics;
import no.ntnu.master.utils.Settings;


/**
 * Created by B on 14.12.2015.
 * @version 1.0
 * A screen for changing settings related to the behaviour of the application
 */
public class ScreenSettings extends ScreenAdapter {

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;

    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;

    /**
     * Variables used to create the user interface
     */
    private Label screenHeader, screenWidthLabel, screenHeightLabel, screenWidthPixelsLabel, screenHeightPixelsLabel, screenViewingDistanceLabel;
    private TextField screenWidth, screenHeight, screenWidthPixels, screenHeightPixels, screenViewingDistance;
    private Label patchWidthLabel, patchHeightLabel, patchGapLabel;
    private TextField patchWidthField, patchHeightField, patchGapField;
    private TextButton buttonDone, buttonCancel;

    /**
     * Label text
     */
    private final String SCREEN_WIDTHM_TEXT = "Width in mm";
    private final String SCREEN_HEIGHTM_TEXT = "Height in mm";
    private final String SCREEN_WIDTHP_TEXT = "Width in pixels";
    private final String SCREEN_HEIGHTP_TEXT = "Height in pixels";
    private final String SCREEN_DISTANCE_TEXT = "Distance in mm";
    private final String SCREEN_HEADER_TEXT = "Screen settings";
    private final String PATCH_WIDTH_TEXT = "Patch width in" + (char) 0x00B0;
    private final String PATCH_HEIGHT_TEXT = "Patch height in" + (char) 0x00B0;
    private final String PATCH_GAP_TEXT = "Patch gap in" + (char) 0x00B0;

    /**
     * Button text
     */
    private final String BUTTON_DONE_TEXT = "Done";
    private final String BUTTON_APPLY_TEXT = "Apply";
    private final String BUTTON_CANCEL_TEXT = "Cancel";
    private final String BUTTON_EXIT_TEXT = "Exit";


    /**
     * Variables for showing the patches
     */
    private Rectangle leftRect, rightRect;
    private ShapeRenderer shapeRenderer;

    /**
     * A stage and table for holding the various user interface elements
     */
    private Table table;
    private Stage stage;
    private ScrollPane scrollPane;

    /**
     * Is there a previous screen
     */
    private boolean hasPrevScreen;

    /**
     * Reference to the game object
     * Used for changing screens
     */
    private DyslexiaDetection game;

    /**
     * Creates a new Settings screen with no previous screen.
     * @param game Reference for the game object
     */
    public ScreenSettings(DyslexiaDetection game) {
        this(game, false);
    }

    /**
     * Creates a new Settings screen with or without a previous screen.
     * @param game Reference for the game object
     * @param hasPrevScreen Set to true if this is called from another screen
     */
    public ScreenSettings(DyslexiaDetection game, boolean hasPrevScreen) {
        this.game = game;
        this.hasPrevScreen = hasPrevScreen;

        createPatches();
        initScreen();
        initUiElements();
        buildUi();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initScreen() {
        //setup a camera to show the screen
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();
        scrollPane = new ScrollPane(table, uiSkin);

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create the gui elements
     */
    private void initUiElements() {
        screenHeader = new Label(SCREEN_HEADER_TEXT, uiSkin);

        screenWidthLabel = new Label(SCREEN_WIDTHM_TEXT, uiSkin);
        screenWidth = new TextField(""+Settings.getScreenWidth(), uiSkin);
        screenWidth.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        screenWidth.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getScreenWidth())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        screenHeightLabel = new Label(SCREEN_HEIGHTM_TEXT, uiSkin);
        screenHeight = new TextField(""+Settings.getScreenHeight(), uiSkin);
        screenHeight.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        screenHeight.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getScreenHeight())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        screenWidthPixelsLabel = new Label(SCREEN_WIDTHP_TEXT, uiSkin);
        screenWidthPixels = new TextField(""+Settings.getScreenWidthPixels(), uiSkin);
        screenWidthPixels.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        screenWidthPixels.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getScreenWidthPixels())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        screenHeightPixelsLabel = new Label(SCREEN_HEIGHTP_TEXT, uiSkin);
        screenHeightPixels = new TextField(""+Settings.getScreenHeightPixels(), uiSkin);
        screenHeightPixels.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        screenHeightPixels.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getScreenHeightPixels())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        screenViewingDistanceLabel = new Label(SCREEN_DISTANCE_TEXT, uiSkin);
        screenViewingDistance = new TextField(""+Settings.getScreenViewingDistance(), uiSkin);
        screenViewingDistance.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        screenViewingDistance.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getScreenViewingDistance())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });


        patchWidthLabel = new Label(PATCH_WIDTH_TEXT, uiSkin);
        patchWidthField = new TextField(""+Settings.getPatchWidth(), uiSkin);
        patchWidthField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        patchWidthField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getPatchWidth())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        patchHeightLabel = new Label(PATCH_HEIGHT_TEXT, uiSkin);
        patchHeightField = new TextField(""+Settings.getPatchHeight(), uiSkin);
        patchHeightField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        patchHeightField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getPatchHeight())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        patchGapLabel = new Label(PATCH_GAP_TEXT, uiSkin);
        patchGapField = new TextField(""+Settings.getPatchGap(), uiSkin);
        patchGapField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        patchGapField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    int i = Integer.parseInt(textField.getText());
                    if (i != (Settings.getPatchGap())) {
                        buttonDone.setText(BUTTON_APPLY_TEXT);
                    } else buttonDone.setText(BUTTON_DONE_TEXT);
                }
            }
        });

        buttonDone = new TextButton(BUTTON_DONE_TEXT, uiSkin);
        buttonDone.setColor(Color.GREEN);
        buttonDone.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                switch (buttonDone.getText().toString()) {
                    case BUTTON_APPLY_TEXT:
                        storeContent();
                        createPatches();
                        buttonDone.setText(BUTTON_DONE_TEXT);
                        break;
                    case BUTTON_DONE_TEXT:
                        if (hasPrevScreen) {
                            Gdx.input.setOnscreenKeyboardVisible(false);
                            game.setScreen(new SettingsMenu(game));
                            break;
                        } else {
                            Gdx.input.setOnscreenKeyboardVisible(false);
                            game.setScreen(new MainMenu(game));
                            break;
                        }
                }
            }
        });


        if (hasPrevScreen){
            buttonCancel = new TextButton(BUTTON_CANCEL_TEXT,uiSkin);
            buttonCancel.setColor(Color.ORANGE);
        } else {
            buttonCancel = new TextButton(BUTTON_EXIT_TEXT, uiSkin);
            buttonCancel.setColor(Color.valueOf("FF3333"));
        }

        buttonCancel.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                switch (buttonCancel.getText().toString()) {
                    case BUTTON_CANCEL_TEXT:
                        Gdx.input.setOnscreenKeyboardVisible(false);
                        game.setScreen(new SettingsMenu(game));
                        break;
                    case BUTTON_EXIT_TEXT:
                        Gdx.input.setOnscreenKeyboardVisible(false);
                        Gdx.app.exit();
                }
            }
        });
    }

    /**
     * Populate the table with the UI elements
     */
    private void buildUi() {

        float prefW = (float)Math.ceil(Gdx.graphics.getWidth() * 140 / 1280);

        scrollPane.setFillParent(true);

        //Simple fix for onScreenKeyboard covering input fields
        table.padBottom(cam.viewportHeight / 2);

        table.add(screenHeader).colspan(2).center().pad(20f).row();

        table.add(screenWidthLabel).left();
        table.add(screenHeightLabel).right().row();

        table.add(screenWidth).prefWidth(prefW).padBottom(5f).left();
        table.add(screenHeight).prefWidth(prefW).padBottom(5f).right().row();

        table.add(screenWidthPixelsLabel).left();
        table.add(screenHeightPixelsLabel).right().row();

        table.add(screenWidthPixels).prefWidth(prefW).padBottom(5f).left();
        table.add(screenHeightPixels).prefWidth(prefW).padBottom(5f).right().row();

        table.add(screenViewingDistanceLabel).left().padRight(40f).row();
        table.add(screenViewingDistance).prefWidth(prefW).padBottom(5f).left().row();

        table.row();

        table.add(patchWidthLabel).left();
        table.add(patchHeightLabel).right().row();

        table.add(patchWidthField).prefWidth(prefW).padBottom(5f).left();
        table.add(patchHeightField).prefWidth(prefW).padBottom(5f).right().row();

        table.add(patchGapLabel).prefWidth(prefW).left().row();
        table.add(patchGapField).prefWidth(prefW).padBottom(5f).left().row();

        table.add(buttonDone).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5f).padBottom(5f).row();

        table.add(buttonCancel).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5f).padBottom(5f).row();

        stage.addActor(scrollPane);
    }


    /**
     * Calculate patch sizes
     */
    private void createPatches() {
        shapeRenderer = new ShapeRenderer();

        //Create rectangles to show the patches
        float patchSeparation = Psychophysics.getPatchSeparationInPixels();
        float patchWidth = Psychophysics.getPatchWidthInPixels();
        float patchHeight = Psychophysics.getPatchHeightInPixels();

        float screenXCenter = Settings.getScreenWidthPixels()/2;
        float screenYCenter = Settings.getScreenHeightPixels()/2;

        leftRect = new Rectangle(screenXCenter-patchWidth-(patchSeparation/2), screenYCenter-(patchHeight/2), patchWidth, patchHeight);
        rightRect = new Rectangle(leftRect.x+patchWidth+patchSeparation, screenYCenter-(patchHeight/2), patchWidth ,patchHeight);
    }

    /**
     * Saves the values from the textFields into settings when "Done" is pushed.
     * Quick fix for invalid input
     */
    private void storeContent() {
        try {

            Settings.setScreenWidth(Integer.parseInt(screenWidth.getText()));
            Settings.setScreenHeight(Integer.parseInt(screenHeight.getText()));

            Settings.setScreenWidthPixels(Integer.parseInt(screenWidthPixels.getText()));
            Settings.setScreenHeightPixels(Integer.parseInt(screenHeightPixels.getText()));

            Settings.setScreenViewingDistance(Integer.parseInt(screenViewingDistance.getText()));

            Settings.setPatchWidth(Integer.parseInt(patchWidthField.getText()));
            Settings.setPatchHeight(Integer.parseInt(patchHeightField.getText()));
            Settings.setPatchGap(Integer.parseInt(patchGapField.getText()));

            if(Settings.getFullScreenFlag()) {
                Gdx.graphics.setDisplayMode(
                        Integer.parseInt(screenWidthPixels.getText()),
                        Integer.parseInt(screenHeightPixels.getText()),
                        true
                );
            }

        } catch (NumberFormatException exp ) {
            Gdx.app.log("ScreenSettings", "This should never happen: " + exp);
        }

    }


    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        if (Gdx.input.isKeyPressed(Input.Keys.BACK) || Gdx.input.isKeyJustPressed(Settings.getInputKeyBack())) {
            game.backPushed = true;
            game.setScreen(new SettingsMenu(game));
        }

        shapeRenderer.begin(ShapeRenderer.ShapeType.Line);
        shapeRenderer.setColor(Color.WHITE);
        shapeRenderer.rect(leftRect.x, leftRect.y, leftRect.width, leftRect.height);
        shapeRenderer.rect(rightRect.x, rightRect.y, rightRect.width, rightRect.height);
        shapeRenderer.end();

        stage.act();
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(viewport.getScreenWidth() / 2);
        table.setY(viewport.getScreenHeight() / 2);
    }

    @Override
    public void pause() {
        storeContent();
    }

    @Override
    public void dispose() {
        super.dispose();
        storeContent();
        uiSkin.dispose();
    }
}
