package no.ntnu.master.settingcreens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.ntnu.master.DyslexiaDetection;
import no.ntnu.master.menus.SettingsMenu;
import no.ntnu.master.utils.AssetLoader;
import no.ntnu.master.utils.Settings;

/**
 * Created by B on 06.01.2016.
 * Screen for editing settings related to the staircase procedure
 */
public class StaircaseSettings extends ScreenAdapter {

    /**
     * Orthographic camera to display 2D content
     */
    private OrthographicCamera cam;

    private Viewport viewport;

    /**
     * Variable for holding the skins to the user interface
     */
    private Skin uiSkin = AssetLoader.uiSkin;

    /**
     * Variables used to create the user interface
     */
    private Label screenHeader;
    private Label correctAnswerLabel;
    private Label wrongAnswerLabel;
    private Label numberOfTriesLabel;
    private Label reversalPointsLabel;
    private Label reversalPointsToUseLabel;
    private TextField correctAnswerField;
    private TextField wrongAnswerField;
    private TextField numberOfTriesField;
    private TextField reversalPointsField;
    private TextField reversalPointsToUseField;
    private TextButton buttonDone, buttonCancel;

    /**
     * Label text
     */
    private final String SCREEN_HEADER_TEXT = "Staircase Settings";
    private final String CORRECT_ANSWER_TEXT = "On correct, lower dB by: ";
    private final String WRONG_ANSWER_TEXT = "On incorrect, increase dB by: ";
    private final String NUMBER_OF_TRIES_TEXT = "Maximum number of tries: ";
    private final String REVERSAL_POINTS_TEXT = "Number of reversal points: ";
    private final String REVERSAL_TO_USE_TEXT = "Calculate mean from last: ";

    /**
     * Button text
     */
    private final String BUTTON_DONE_TEXT = "Done";
    private final String BUTTON_CANCEL_TEXT = "Cancel";

    /**
     * A stage and table for holding the various user interface elements
     */
    private Table table;
    private Stage stage;
    private ScrollPane scrollPane;

    /**
     * Reference to the game object
     * Used for changing screens
     */
    private DyslexiaDetection game;

    public StaircaseSettings(DyslexiaDetection game) {
        this.game = game;
        initScreen();
        initUiElements();
        buildUi();
    }

    /**
     * Creates a camera and sets the viewport for the camera and stage
     */
    private void initScreen() {
        //setup a camera to show the screen
        cam = new OrthographicCamera();
        viewport = new ScreenViewport(cam);
        viewport.apply();
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);

        //stage and table for ui elements
        stage = new Stage(viewport, game.batch);
        table = new Table();
        scrollPane = new ScrollPane(table, uiSkin);

        //Use the stage for input processing
        Gdx.input.setInputProcessor(stage);
    }

    /**
     * Create and fill the stage with the gui elements
     */
    private void initUiElements() {

        screenHeader = new Label(SCREEN_HEADER_TEXT, uiSkin);

        correctAnswerLabel = new Label(CORRECT_ANSWER_TEXT, uiSkin);
        correctAnswerField = new TextField(""+Settings.getStaircaseCorrectDB(), uiSkin);
        correctAnswerField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.-]"," +"));
                Matcher matcher = pattern.matcher("0123456789.-");
                return matcher.find();
            }

        });

        wrongAnswerLabel = new Label(WRONG_ANSWER_TEXT, uiSkin);
        wrongAnswerField = new TextField(""+Settings.getStaircaseWrongDB(), uiSkin);
        wrongAnswerField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789.-]"," +"));
                Matcher matcher = pattern.matcher("0123456789.-");
                return matcher.find();
            }

        });

        numberOfTriesLabel = new Label(NUMBER_OF_TRIES_TEXT, uiSkin);
        numberOfTriesField = new TextField(""+Settings.getStaircaseMaxTries(), uiSkin);
        numberOfTriesField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]"," +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });

        reversalPointsLabel = new Label(REVERSAL_POINTS_TEXT, uiSkin);
        reversalPointsField = new TextField(""+Settings.getStaircaseReversalPoints(), uiSkin);
        reversalPointsField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]"," +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        reversalPointsField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    float a = Float.parseFloat(textField.getText());
                    float b = Float.parseFloat(reversalPointsToUseField.getText());
                    if (a < b) {
                        reversalPointsToUseField.setText(textField.getText());
                    }
                }
            }
        });

        reversalPointsToUseLabel = new Label(REVERSAL_TO_USE_TEXT, uiSkin);
        reversalPointsToUseField = new TextField(""+Settings.getStaircaseReversalsToUse(), uiSkin);
        reversalPointsToUseField.setTextFieldFilter(new TextField.TextFieldFilter() {
            @Override
            public boolean acceptChar(TextField textField, char c) {
                Pattern pattern = Pattern.compile(String.valueOf(c).replaceAll("[^0123456789]", " +"));
                Matcher matcher = pattern.matcher("0123456789");
                return matcher.find();
            }

        });
        reversalPointsToUseField.setTextFieldListener(new TextField.TextFieldListener() {
            @Override
            public void keyTyped(TextField textField, char c) {
                if (textField.getText().length() > 0) {
                    float a = Float.parseFloat(textField.getText());
                    float b = Float.parseFloat(reversalPointsField.getText());
                    if (a > b) {
                        textField.setText(reversalPointsField.getText());
                    }
                }
            }
        });

        buttonDone = new TextButton(BUTTON_DONE_TEXT, uiSkin);
        buttonDone.setColor(Color.GREEN);
        buttonDone.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.input.setOnscreenKeyboardVisible(false);
                storeContent();
                game.setScreen(new SettingsMenu(game));
            }
        });

        buttonCancel = new TextButton(BUTTON_CANCEL_TEXT, uiSkin);
        buttonCancel.setColor(Color.ORANGE);
        buttonCancel.addListener(new ChangeListener() {
            @Override
            public void changed(ChangeEvent event, Actor actor) {
                Gdx.input.setOnscreenKeyboardVisible(false);
                game.setScreen(new SettingsMenu(game));
            }
        });
    }

    private void buildUi() {

        float prefW = (float)Math.ceil(Gdx.graphics.getWidth() * 140 / 1280);

        scrollPane.setFillParent(true);

        //Simple fix for onScreenKeyboard covering input fields
        table.padBottom(cam.viewportHeight / 2);

        table.add(screenHeader).colspan(2).center().pad(20).row();

        table.add(correctAnswerLabel).left();
        table.add(correctAnswerField).prefWidth(prefW).padBottom(5).padTop(5).row();

        table.add(wrongAnswerLabel).left();
        table.add(wrongAnswerField).prefWidth(prefW).padBottom(5).padTop(5).row();

        table.add(numberOfTriesLabel).left();
        table.add(numberOfTriesField).prefWidth(prefW).padBottom(5).padTop(5).row();

        table.add(reversalPointsLabel).left();
        table.add(reversalPointsField).prefWidth(prefW).padBottom(5).padTop(5).row();

        table.add(reversalPointsToUseLabel).left();
        table.add(reversalPointsToUseField).prefWidth(prefW).padBottom(5).padTop(5).row();

        table.add(buttonDone).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5f).padBottom(5f).row();

        table.add(buttonCancel).colspan(2).fill().height(viewport.getScreenHeight() / 10).
                padTop(5f).padBottom(5f).row();

        stage.addActor(scrollPane);
    }

    private void storeContent() {
        try {

            Settings.setStaircaseCorrectDB(Float.parseFloat(correctAnswerField.getText()));

            Settings.setStaircaseWrongDB(Float.parseFloat(wrongAnswerField.getText()));

            Settings.setStaircaseMaxTires(Integer.parseInt(numberOfTriesField.getText()));

            Settings.setStaircaseReversalPoints(Integer.parseInt(reversalPointsField.getText()));

            Settings.setStaircaseReversalsToUse(Integer.parseInt(reversalPointsToUseField.getText()));

        } catch (NumberFormatException exp) {
            Gdx.app.log("STAIRCASE SETTINGS", "Exception in storing content from fields " + exp);
        }
    }

    /**
     * Render the screen
     * @param delta delta time
     */
    @Override
    public void render(float delta) {
        Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        if (Gdx.input.isKeyPressed(Input.Keys.BACK) || Gdx.input.isKeyJustPressed(Settings.getInputKeyBack())) {
            game.backPushed = true;
            game.setScreen(new SettingsMenu(game));
        }
        stage.act();
        stage.draw();
    }

    /**
     * Called when the window is resized
     * @param width the new window width
     * @param height the new window height
     */
    @Override
    public void resize(int width, int height) {
        viewport.update(width, height);
        cam.position.set(cam.viewportWidth / 2, cam.viewportHeight / 2, 0);
        table.setX(width/2);
        table.setY(height/2);
        //stage.getViewport().update(width, height);
    }

    @Override
    public void pause() {
        storeContent();
    }

    @Override
    public void dispose() {
        storeContent();
        uiSkin.dispose();
    }
}
