package no.ntnu.master.utils;

import com.badlogic.gdx.utils.Array;

/**
 * Created by B on 16.12.2015.
 * Helper class for psychophysical calculations.
 */
public class Psychophysics {

    /**
     * Function for converting pixels to visual angle.
     * res and dim should both use either height or width.
     * @param pixels The amount of pixels.
     * @param distance The viewing distance to the screen in millimeters.
     * @param res The screen height or width in pixels.
     * @param dim The screen height or width in millimeters.
     * @return Visual angle in degrees.
     */
    public static float visualAngleInDegrees(float pixels, float distance, float res, float dim) {
        return (float) Math.toDegrees(Math.atan(((pixels/2)/(distance*res/dim))))*2;
    }

    /**
     * Function for converting visual angle degrees to pixels.
     * res and dim should both use either height or width.
     * @param degrees The visual angle in degrees.
     * @param distance The viewing distance to the screen in millimeters.
     * @param res The screen height or width in pixels.
     * @param dim The screen height or width in millimeters.
     * @return Visual angle in pixels.
     */
    public static float visualAngleInPixels(float degrees, float distance, float res, float dim) {
        return (float) Math.tan(Math.toRadians(degrees / 2))*2*distance*res/dim;
    }

    /**
     * Convert amplitude ratio/fctor to decibel.
     * @param factor The amplitude ratio to be converted.
     * @return Returns the decibel value for given amplitude ratio as a floating point.
     */
    public static float factorToDecibel(float factor) {
        float squared = factor * factor;
        return (float) (10*Math.log10(squared));
    }

    /**
     * Converts decibel to amplitude ratio/factor.
     * @param decibel The decibel to be converted.
     * @return Returns the amplitude ratio for the given decibel as a floating point.
     */
    public static float decibelToFactor(float decibel) {
        return (float) Math.pow(10, (decibel/20));
    }

    /**
     * Calculate the width of the patch.
     * @return The width in pixels
     */
    public static float getPatchWidthInPixels() {
        return visualAngleInPixels(Settings.getPatchWidth(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenWidthPixels(),
                Settings.getScreenWidth());
    }

    /**
     * Calculate the height of the patch.
     * @return The height in pixels
     */
    public static float getPatchHeightInPixels() {
        return visualAngleInPixels(Settings.getPatchHeight(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenHeightPixels(),
                Settings.getScreenHeight());
    }

    /**
     * Calculate the gap between the patches.
     * @return The separation in pixels
     */
    public static float getPatchSeparationInPixels() {
        return visualAngleInPixels(Settings.getPatchGap(),
                Settings.getScreenViewingDistance(),
                Settings.getScreenWidthPixels(),
                Settings.getScreenWidth());
    }

    /**
     * Calculate the geometric mean
     * @return The geometric mean
     */
    public static float geometricMean(Array<Float> results, int ofLast) {
        if (results.size == 0) {
            return 0;
        }
        double sum = 1;
        int loops = (results.size>=ofLast) ? (results.size - ofLast) : 0;
        double root = 0;
        for (int i = loops; i < results.size; i++) {
            sum *= results.get(i);
            root++;
        }
        return (float) Math.pow(sum, 1d/root);
    }

}
