package no.ntnu.master.utils;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Preferences;

/**
 * Static class for loading and storing settings related to the behaviour of on screen elements.
 * @author B
 */

public class Settings {

    /**
     * The preference object used for storing and retrieving values.
     */
    private static Preferences prefs = Gdx.app.getPreferences("DyslexiaDetection");

    /**
     * Key values for storing and retrieving parameters to/from a preference object.
     * Physical measures of the screen width and height in mm
     * Pixel count of the screen in width and height
     * The viewing distance of the screen in mm
     */

    //Screen settings
    private static final String SCREEN_WIDTH = "screenWidth";
    private static final String SCREEN_HEIGHT = "screenHeight";
    private static final String SCREEN_WIDTH_PIXELS = "screenWidthPixels";
    private static final String SCREEN_HEIGHT_PIXELS = "screenHeightPixels";
    private static final String SCREEN_VIEWING_DISTANCE = "screenViewingDistance";
    private static final String SCREEN_PATCH_WIDTH = "screenPatchWidth";
    private final static String SCREEN_PATCH_HEIGHT = "screenPatchHeight";
    private final static String SCREEN_PATCH_GAP = "screenPatchGap";

    //Dot behavior settings
    private static final String DOT_MAX_AMOUNT = "dotMaxAmount";
    private static final String DOT_RADIUS_PIXELS = "dotRadiusPixels";
    private static final String DOT_SPACING = "dotSpacing";
    private static final String DOT_VELOCITY = "dotVelocity";
    private static final String DOT_COHERENCY = "dotCoherency";
    private static final String DOT_ANIMATION_TIME = "dotAnimationTime";
    private static final String DOT_MAX_ALIVE_TIME = "dotMaxAliveTime";
    private static final String DOT_TO_KILL = "dotToKill";
    private static final String DOT_HORIZONTAL_REVERSAL_TIME = "dotReversalTime";
    private static final String DOT_RANDOM_DIRECTION_TIME = "dotRandomTime";

    //Form behaviour settings
    private static final String FORM_AUTO_MODE = "formAutoMode";
    private static final String FORM_MAX_AMOUNT = "formMaxAmount";
    private static final String FORM_DIAMETER_WB = "formDiameterWidthBased";
    private static final String FORM_CIRCLES = "numberOfCircles";
    private static final String FORM_CIRCLES_GAP = "circleGap";
    private static final String FORM_LINE_LENGTH = "formLineLength";
    private static final String FORM_LINE_HEIGHT = "formLineHeight";
    private static final String FORM_LINE_GAP = "formLineGap";
    private static final String FORM_COHERENCY = "formCoherency";
    private static final String FORM_F_DETECTION_TIME = "formFixedDetectionTime";
    private static final String FORM_R_DETECTION_TIME = "formRandomDetectionTime";

    //Staircase settings
    private static final String STAIR_CORRECT_ANSWER_DB = "stairCorrectDB";
    private static final String STAIR_WRONG_ANSWER_DB = "stairWrongDB";
    private static final String STAIR_MAX_TRIES = "stairMaxTries";
    private static final String STAIR_REVERSAL_POINTS = "stairReversalPoints";
    private static final String STAIR_REVERSALS_TO_USE = "stairReversalsToUse";

    //Input settings
    private static final String INPUT_KEY_LEFT = "inputKeyLeft";
    private static final String INPUT_KEY_RIGHT = "inputKeyRight";
    private static final String INPUT_KEY_BACK = "inputKeyBack";
    private static final String INPUT_CONTINUOUS_MODE = "continuousMode";

    /**
     * Default values for the settings
     */

    //Screen default values
    private static final int SCREEN_W_MM_DEF_VALUE =
            (Gdx.app.getType().equals(Application.ApplicationType.Desktop)) ?
                    (int)(Gdx.graphics.getDesktopDisplayMode().width / (Gdx.graphics.getPpcX() / 10)) :
                    (int)(Gdx.graphics.getWidth() / (Gdx.graphics.getPpcX() / 10));

    private static final int SCREEN_H_MM_DEF_VALUE =
            (Gdx.app.getType().equals(Application.ApplicationType.Desktop)) ?
                    (int)(Gdx.graphics.getDesktopDisplayMode().height / (Gdx.graphics.getPpcY() / 10)) :
                    (int)(Gdx.graphics.getHeight() / (Gdx.graphics.getPpcY() / 10));

    private static final int SCREEN_W_PX_DEF_VALUE =
        (Gdx.app.getType().equals(Application.ApplicationType.Desktop)) ?
                Gdx.graphics.getDesktopDisplayMode().width :
                (Gdx.graphics.getWidth());

    private static final int SCREEN_H_PX_DEF_VALUE =
            (Gdx.app.getType().equals(Application.ApplicationType.Desktop)) ?
                    Gdx.graphics.getDesktopDisplayMode().height :
                    (Gdx.graphics.getHeight());

    private static final int SCREEN_VD_MM_DEF_VALUE = 300;
    private static final int SCREEN_PATCH_WIDTH_DEF_VALUE = 10;
    private static final int SCREEN_PATCH_HEIGHT_DEF_VALUE = 14;
    private static final int SCREEN_PATCH_GAP_DEF_VALUE = 5;

    //Dot behavior default values
    private static final int DOT_MAX_AMOUNT_DEF_VALUE = 300;
    private static final float DOT_RADIUS_PIXELS_DEF_VALUE = 1f;
    private static final float DOT_SPACING_DEF_VALUE = 1f;
    private static final float DOT_VELOCITY_DEF_VALUE = 50f;
    private static final float DOT_COHERENCY_DEF_VALUE = 50f;
    private static final float DOT_ANIMATION_TIME_DEF_VALUE = 5f;
    private static final float DOT_MAX_ALIVE_TIME_DEF_VALUE = 0.085f;
    private static final int DOT_TO_KILL_DEF_VALUE = 10;
    private static final float DOT_HORIZONTAL_REVERSAL_TIME_DEF_VALUE = 0.572f;
    private static final float DOT_RANDOM_DIRECTION_TIME_DEF_VALUE = 0.572f;

    //Form behaviour default values
    private static final boolean FORM_AUTO_MODE_DEF_VALUE = true;
    private static final int FORM_MAX_AMOUNT_DEF_VALUE = 600;
    private static final float FORM_DIAMETER_WB_DEF_VALUE = 8f;
    private static final int FORM_CIRCLES_DEF_VALUE = 4;
    private static final float FORM_CIRCLES_GAP_DEF_VALUE = 0.9f;
    private static final float FORM_LINE_LENGTH_DEF_VALUE = 0.4f;
    private static final float FORM_LINE_HEIGHT_DEF_VALUE = 1f;
    private static final float FORM_LINE_GAP_DEF_VALUE = 0.4f;
    private static final float FORM_COHERENCY_DEF_VALUE = 100f;
    private static final float FORM_F_DETECTION_TIME_DEF_VALUE = 4f;
    private static final float FORM_R_DETECTION_TIME_DEF_VALUE = 1000f;

    //Staircase default values
    private static final float STAIR_CORRECT_ANSWER_DB_DEF_VALUE = 1f;
    private static final float STAIR_WRONG_ANSWER_DB_DEF_VALUE = 3f;
    private static final int STAIR_MAX_TRIES_DEF_VALUE = 100;
    private static final int STAIR_REVERSAL_POINTS_DEF_VALUE = 10;
    private static final int STAIR_REVERSALS_TO_USE_DEF_VALUE = 8;

    //Input default values
    private static final int INPUT_KEY_LEFT_DEF_VALUE = Input.Keys.S;
    private static final int INPUT_KEY_RIGHT_DEF_VALUE = Input.Keys.L;
    private static final int INPUT_KEY_BACK_DEF_VALUE = Input.Keys.ESCAPE;
    private static final boolean INPUT_CONTINUOUS_MODE_DEF_VALUE = false;

    /**
     * Loads default or stored values from a preference object.
     */
    public static void load() {

        if (!prefs.contains(SCREEN_WIDTH)) {
            prefs.putInteger(SCREEN_WIDTH, SCREEN_W_MM_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_HEIGHT)) {
            prefs.putInteger(SCREEN_HEIGHT, SCREEN_H_MM_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_WIDTH_PIXELS)) {
            prefs.putInteger(SCREEN_WIDTH_PIXELS, SCREEN_W_PX_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_HEIGHT_PIXELS)) {
            prefs.putInteger(SCREEN_HEIGHT_PIXELS, SCREEN_H_PX_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_VIEWING_DISTANCE)) {
            prefs.putInteger(SCREEN_VIEWING_DISTANCE, SCREEN_VD_MM_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_PATCH_WIDTH)) {
            prefs.putInteger(SCREEN_PATCH_WIDTH, SCREEN_PATCH_WIDTH_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_PATCH_HEIGHT)) {
            prefs.putInteger(SCREEN_PATCH_HEIGHT, SCREEN_PATCH_HEIGHT_DEF_VALUE);
        }

        if (!prefs.contains(SCREEN_PATCH_GAP)) {
            prefs.putInteger(SCREEN_PATCH_GAP, SCREEN_PATCH_GAP_DEF_VALUE);
        }

        if (!prefs.contains(DOT_MAX_AMOUNT)) {
            prefs.putInteger(DOT_MAX_AMOUNT, DOT_MAX_AMOUNT_DEF_VALUE);
        }

        if (!prefs.contains(DOT_RADIUS_PIXELS)) {
            prefs.putFloat(DOT_RADIUS_PIXELS, DOT_RADIUS_PIXELS_DEF_VALUE);
        }

        if (!prefs.contains(DOT_SPACING)) {
            prefs.putFloat(DOT_SPACING, DOT_SPACING_DEF_VALUE);
        }

        if (!prefs.contains(DOT_VELOCITY)) {
            prefs.putFloat(DOT_VELOCITY, DOT_VELOCITY_DEF_VALUE);
        }

        if (!prefs.contains(DOT_COHERENCY)) {
            prefs.putFloat(DOT_COHERENCY, DOT_COHERENCY_DEF_VALUE);
        }

        if (!prefs.contains(DOT_ANIMATION_TIME)) {
            prefs.putFloat(DOT_ANIMATION_TIME, DOT_ANIMATION_TIME_DEF_VALUE);
        }

        if (!prefs.contains(DOT_MAX_ALIVE_TIME)) {
            prefs.putFloat(DOT_MAX_ALIVE_TIME, DOT_MAX_ALIVE_TIME_DEF_VALUE);
        }

        if (!prefs.contains(DOT_TO_KILL)) {
            prefs.putInteger(DOT_TO_KILL, DOT_TO_KILL_DEF_VALUE);
        }

        if (!prefs.contains(DOT_HORIZONTAL_REVERSAL_TIME)) {
            prefs.putFloat(DOT_HORIZONTAL_REVERSAL_TIME, DOT_HORIZONTAL_REVERSAL_TIME_DEF_VALUE);
        }

        if (!prefs.contains(DOT_RANDOM_DIRECTION_TIME)) {
            prefs.putFloat(DOT_RANDOM_DIRECTION_TIME, DOT_RANDOM_DIRECTION_TIME_DEF_VALUE);
        }

        if (!prefs.contains(FORM_AUTO_MODE)) {
            prefs.putBoolean(FORM_AUTO_MODE, FORM_AUTO_MODE_DEF_VALUE);
        }

        if (!prefs.contains(FORM_MAX_AMOUNT)) {
            prefs.putInteger(FORM_MAX_AMOUNT, FORM_MAX_AMOUNT_DEF_VALUE);
        }

        if (!prefs.contains(FORM_DIAMETER_WB)) {
            prefs.putFloat(FORM_DIAMETER_WB, FORM_DIAMETER_WB_DEF_VALUE);
        }

        if(!prefs.contains(FORM_CIRCLES)) {
            prefs.putInteger(FORM_CIRCLES, FORM_CIRCLES_DEF_VALUE);
        }

        if (!prefs.contains(FORM_CIRCLES_GAP)) {
            prefs.putFloat(FORM_CIRCLES_GAP, FORM_CIRCLES_GAP_DEF_VALUE);
        }

        if (!prefs.contains(FORM_LINE_HEIGHT)) {
            prefs.putFloat(FORM_LINE_HEIGHT, FORM_LINE_HEIGHT_DEF_VALUE);
        }

        if (!prefs.contains(FORM_LINE_LENGTH)) {
            prefs.putFloat(FORM_LINE_LENGTH, FORM_LINE_LENGTH_DEF_VALUE);
        }

        if(!prefs.contains(FORM_LINE_GAP)) {
            prefs.putFloat(FORM_LINE_GAP, FORM_LINE_GAP_DEF_VALUE);
        }

        if (!prefs.contains(FORM_COHERENCY)) {
            prefs.putFloat(FORM_COHERENCY, FORM_COHERENCY_DEF_VALUE);
        }

        if (!prefs.contains(FORM_F_DETECTION_TIME)) {
            prefs.putFloat(FORM_F_DETECTION_TIME, FORM_F_DETECTION_TIME_DEF_VALUE);
        }

        if (!prefs.contains(FORM_R_DETECTION_TIME)) {
            prefs.putFloat(FORM_R_DETECTION_TIME, FORM_R_DETECTION_TIME_DEF_VALUE);
        }

        if (!prefs.contains(STAIR_CORRECT_ANSWER_DB)) {
            prefs.putFloat(STAIR_CORRECT_ANSWER_DB, STAIR_CORRECT_ANSWER_DB_DEF_VALUE);
        }

        if (!prefs.contains(STAIR_WRONG_ANSWER_DB)) {
            prefs.putFloat(STAIR_WRONG_ANSWER_DB, STAIR_WRONG_ANSWER_DB_DEF_VALUE);
        }

        if (!prefs.contains(STAIR_MAX_TRIES)) {
            prefs.putInteger(STAIR_MAX_TRIES, STAIR_MAX_TRIES_DEF_VALUE);
        }

        if (!prefs.contains(STAIR_REVERSAL_POINTS)) {
            prefs.putInteger(STAIR_REVERSAL_POINTS, STAIR_REVERSAL_POINTS_DEF_VALUE);
        }

        if (!prefs.contains(STAIR_REVERSALS_TO_USE)) {
            prefs.putInteger(STAIR_REVERSALS_TO_USE, STAIR_REVERSALS_TO_USE_DEF_VALUE);
        }

        if(!prefs.contains(INPUT_KEY_LEFT)) {
            prefs.putInteger(INPUT_KEY_LEFT, INPUT_KEY_LEFT_DEF_VALUE);
        }

        if(!prefs.contains(INPUT_KEY_RIGHT)) {
            prefs.putInteger(INPUT_KEY_RIGHT, INPUT_KEY_RIGHT_DEF_VALUE);
        }

        if(!prefs.contains(INPUT_KEY_BACK)) {
            prefs.putInteger(INPUT_KEY_BACK, INPUT_KEY_BACK_DEF_VALUE);
        }

        if(!prefs.contains(INPUT_CONTINUOUS_MODE)) {
            prefs.putBoolean(INPUT_CONTINUOUS_MODE, INPUT_CONTINUOUS_MODE_DEF_VALUE);
        }

        prefs.flush();
    }

    public static void resetAllSettings() {
        prefs.clear();

        prefs.putInteger(SCREEN_WIDTH, SCREEN_W_MM_DEF_VALUE);

        prefs.putInteger(SCREEN_HEIGHT, SCREEN_H_MM_DEF_VALUE);

        prefs.putInteger(SCREEN_WIDTH_PIXELS, SCREEN_W_PX_DEF_VALUE);

        prefs.putInteger(SCREEN_HEIGHT_PIXELS, SCREEN_H_PX_DEF_VALUE);

        prefs.putInteger(SCREEN_VIEWING_DISTANCE, SCREEN_VD_MM_DEF_VALUE);

        prefs.putInteger(SCREEN_PATCH_WIDTH, SCREEN_PATCH_WIDTH_DEF_VALUE);

        prefs.putInteger(SCREEN_PATCH_HEIGHT, SCREEN_PATCH_HEIGHT_DEF_VALUE);

        prefs.putInteger(SCREEN_PATCH_GAP, SCREEN_PATCH_GAP_DEF_VALUE);

        prefs.putInteger(DOT_MAX_AMOUNT, DOT_MAX_AMOUNT_DEF_VALUE);

        prefs.putFloat(DOT_RADIUS_PIXELS, DOT_RADIUS_PIXELS_DEF_VALUE);

        prefs.putFloat(DOT_SPACING, DOT_SPACING_DEF_VALUE);

        prefs.putFloat(DOT_VELOCITY, DOT_VELOCITY_DEF_VALUE);

        prefs.putFloat(DOT_COHERENCY, DOT_COHERENCY_DEF_VALUE);

        prefs.putFloat(DOT_ANIMATION_TIME, DOT_ANIMATION_TIME_DEF_VALUE);

        prefs.putFloat(DOT_MAX_ALIVE_TIME, DOT_MAX_ALIVE_TIME_DEF_VALUE);

        prefs.putInteger(DOT_TO_KILL, DOT_TO_KILL_DEF_VALUE);

        prefs.putBoolean(FORM_AUTO_MODE, FORM_AUTO_MODE_DEF_VALUE);

        prefs.putFloat(DOT_HORIZONTAL_REVERSAL_TIME, DOT_HORIZONTAL_REVERSAL_TIME_DEF_VALUE);

        prefs.putFloat(DOT_RANDOM_DIRECTION_TIME, DOT_RANDOM_DIRECTION_TIME_DEF_VALUE);

        prefs.putInteger(FORM_MAX_AMOUNT, FORM_MAX_AMOUNT_DEF_VALUE);

        prefs.putFloat(FORM_DIAMETER_WB, FORM_DIAMETER_WB_DEF_VALUE);

        prefs.putInteger(FORM_CIRCLES, FORM_CIRCLES_DEF_VALUE);

        prefs.putFloat(FORM_CIRCLES_GAP, FORM_CIRCLES_GAP_DEF_VALUE);

        prefs.putFloat(FORM_LINE_LENGTH, FORM_LINE_LENGTH_DEF_VALUE);

        prefs.putFloat(FORM_LINE_HEIGHT, FORM_LINE_HEIGHT_DEF_VALUE);

        prefs.putFloat(FORM_LINE_GAP, FORM_LINE_GAP_DEF_VALUE);

        prefs.putFloat(FORM_COHERENCY, FORM_COHERENCY_DEF_VALUE);

        prefs.putFloat(FORM_F_DETECTION_TIME, FORM_F_DETECTION_TIME_DEF_VALUE);

        prefs.putFloat(FORM_R_DETECTION_TIME, FORM_R_DETECTION_TIME_DEF_VALUE);

        prefs.putFloat(STAIR_CORRECT_ANSWER_DB, STAIR_CORRECT_ANSWER_DB_DEF_VALUE);

        prefs.putFloat(STAIR_WRONG_ANSWER_DB, STAIR_WRONG_ANSWER_DB_DEF_VALUE);

        prefs.putInteger(STAIR_MAX_TRIES, STAIR_MAX_TRIES_DEF_VALUE);

        prefs.putInteger(STAIR_REVERSAL_POINTS, STAIR_REVERSAL_POINTS_DEF_VALUE);

        prefs.putInteger(STAIR_REVERSALS_TO_USE, STAIR_REVERSALS_TO_USE_DEF_VALUE);

        prefs.putInteger(INPUT_KEY_LEFT, INPUT_KEY_LEFT_DEF_VALUE);

        prefs.putInteger(INPUT_KEY_RIGHT, INPUT_KEY_RIGHT_DEF_VALUE);

        prefs.putInteger(INPUT_KEY_BACK, INPUT_KEY_BACK_DEF_VALUE);

        prefs.putBoolean(INPUT_CONTINUOUS_MODE, INPUT_CONTINUOUS_MODE_DEF_VALUE);

        prefs.flush();
    }

    //Screen setters and getters
    public static int getScreenWidth() {
        return prefs.getInteger(SCREEN_WIDTH);
    }

    public static void setScreenWidth(int i) {
        prefs.putInteger(SCREEN_WIDTH, i);
        prefs.flush();
    }

    public static int getScreenHeight() {
        return prefs.getInteger(SCREEN_HEIGHT);
    }

    public static void setScreenHeight(int i) {
        prefs.putInteger(SCREEN_HEIGHT, i);
        prefs.flush();
    }

    public static int getScreenWidthPixels() {
        return prefs.getInteger(SCREEN_WIDTH_PIXELS);
    }

    public static void setScreenWidthPixels(int i) {
        prefs.putInteger(SCREEN_WIDTH_PIXELS, i);
        prefs.flush();
    }

    public static int getScreenHeightPixels() {
        return prefs.getInteger(SCREEN_HEIGHT_PIXELS);
    }

    public static void setScreenHeightPixels(int i) {
        prefs.putInteger(SCREEN_HEIGHT_PIXELS, i);
        prefs.flush();
    }

    public static int getScreenViewingDistance() {
        return prefs.getInteger(SCREEN_VIEWING_DISTANCE);
    }

    public static void setScreenViewingDistance(int i) {
        prefs.putInteger(SCREEN_VIEWING_DISTANCE, i);
        prefs.flush();
    }

    public static int getPatchWidth() {
        return prefs.getInteger(SCREEN_PATCH_WIDTH);
    }

    public static void setPatchWidth(int i) {
        prefs.putInteger(SCREEN_PATCH_WIDTH, i);
        prefs.flush();
    }

    public static int getPatchHeight() {
        return prefs.getInteger(SCREEN_PATCH_HEIGHT);
    }

    public static void setPatchHeight(int i) {
        prefs.putInteger(SCREEN_PATCH_HEIGHT, i);
        prefs.flush();
    }

    public static int getPatchGap() {
        return prefs.getInteger(SCREEN_PATCH_GAP);
    }

    public static void setPatchGap(int i) {
        prefs.putInteger(SCREEN_PATCH_GAP, i);
        prefs.flush();
    }


    //Dot behaviour setters and getters
    public static int getDotMaxAmount() {
        return prefs.getInteger(DOT_MAX_AMOUNT);
    }

    public static void setDotMaxAmount(int i) {
        prefs.putInteger(DOT_MAX_AMOUNT, i);
        prefs.flush();
    }

    public static float getDotRadiusPixels() {
        return prefs.getFloat(DOT_RADIUS_PIXELS);
    }

    public static void setDotRadiusPixels(float f) {
        prefs.putFloat(DOT_RADIUS_PIXELS, f);
        prefs.flush();
    }

    public static float getDotSpacing() {
        return prefs.getFloat(DOT_SPACING);
    }

    public static void setDotSpacing(float f) {
        prefs.putFloat(DOT_SPACING, f);
        prefs.flush();
    }

    public static float getDotVelocity() {
        return prefs.getFloat(DOT_VELOCITY);
    }

    public static void setDotVelocity(float f) {
        prefs.putFloat(DOT_VELOCITY, f);
        prefs.flush();
    }

    public static float getDotCoherency() {
        return prefs.getFloat(DOT_COHERENCY);
    }

    public static void setDotCoherency(float f) {
        prefs.putFloat(DOT_COHERENCY, f);
        prefs.flush();
    }

    public static float getDotAnimationTime() {
        return prefs.getFloat(DOT_ANIMATION_TIME);
    }

    public static void setDotAnimationTime(float f) {
        prefs.putFloat(DOT_ANIMATION_TIME, f);
        prefs.flush();
    }

    public static float getDotMaxLifeTime() {
        return prefs.getFloat(DOT_MAX_ALIVE_TIME);
    }

    public static void setDotMaxLifeTime(float f) {
        prefs.putFloat(DOT_MAX_ALIVE_TIME, f);
        prefs.flush();
    }

    public static int getDotPercentageToKill() {
        return prefs.getInteger(DOT_TO_KILL);
    }

    public static void setDotPercentageToKill(int i) {
        prefs.putInteger(DOT_TO_KILL, i);
        prefs.flush();
    }

    public static float getDotHorizontalReversalTime() {
        return prefs.getFloat(DOT_HORIZONTAL_REVERSAL_TIME);
    }

    public static void setDotHorizontalReversalTime(float f) {
        prefs.putFloat(DOT_HORIZONTAL_REVERSAL_TIME, f);
        prefs.flush();
    }

    public static float getDotRandomDirectionTime() {
        return prefs.getFloat(DOT_RANDOM_DIRECTION_TIME);
    }

    public static void setDotRandomDirectionTime(float f) {
        prefs.putFloat(DOT_RANDOM_DIRECTION_TIME, f);
        prefs.flush();
    }


    //Form setters and getters

    public static boolean getFormAutoMode() { return prefs.getBoolean(FORM_AUTO_MODE); }
    public static void setFormAutoMode(boolean b) { prefs.putBoolean(FORM_AUTO_MODE, b); }

    public static int getFormMaxLineSegments() {
        return prefs.getInteger(FORM_MAX_AMOUNT);
    }
    public static void setFormMaxLineSegments(int i) {
        prefs.putInteger(FORM_MAX_AMOUNT, i);
        prefs.flush();
    }

    public static float getFormDiameterWb() {
        return prefs.getFloat(FORM_DIAMETER_WB);
    }
    public static void setFormDiameterWb(float f) {
        prefs.putFloat(FORM_DIAMETER_WB, f);
        prefs.flush();
    }

    public static int getFormNrOfCircles() { return prefs.getInteger(FORM_CIRCLES); }
    public static void setFormNrOfCircles(int i) {
        prefs.putInteger(FORM_CIRCLES, i);
        prefs.flush();
    }

    public static float getFormCirclesGap() { return prefs.getFloat(FORM_CIRCLES_GAP); }
    public static void setFormCirclesGap(float f) {
        prefs.putFloat(FORM_CIRCLES_GAP, f);
        prefs.flush();
    }

    public static float getFormLineLength() {
        return prefs.getFloat(FORM_LINE_LENGTH);
    }

    public static void setFormLineLength(float f) {
        prefs.putFloat(FORM_LINE_LENGTH, f);
        prefs.flush();
    }

    public static float getFormLineHeight() {
        return prefs.getFloat(FORM_LINE_HEIGHT);
    }

    public static void setFormLineHeight(float f) {
        prefs.putFloat(FORM_LINE_HEIGHT, f);
        prefs.flush();
    }

    public static float getFormLineGap() { return prefs.getFloat(FORM_LINE_GAP); }
    public static void setFormLineGap(float f) {
        prefs.putFloat(FORM_LINE_GAP, f);
        prefs.flush();
    }

    public static float getFormCoherency() {
        return prefs.getFloat(FORM_COHERENCY);
    }

    public static void setFormCoherency(float f) {
        prefs.putFloat(FORM_COHERENCY, f);
        prefs.flush();
    }

    public static float getFormFDetectionTime() {
        return prefs.getFloat(FORM_F_DETECTION_TIME);
    }

    public static void setFormFDetectionTime(float f) {
        prefs.putFloat(FORM_F_DETECTION_TIME, f);
        prefs.flush();
    }

    public static float getFormRDetectionTime() {
        return prefs.getFloat(FORM_R_DETECTION_TIME);
    }

    public static void setFormRDetectionTime(float f) {
        prefs.putFloat(FORM_R_DETECTION_TIME, f);
        prefs.flush();
    }

    //Staircase setters and getters
    public static float getStaircaseCorrectDB() {
        return prefs.getFloat(STAIR_CORRECT_ANSWER_DB);
    }

    public static void setStaircaseCorrectDB(float f) {
        prefs.putFloat(STAIR_CORRECT_ANSWER_DB, f);
        prefs.flush();
    }

    public static float getStaircaseWrongDB() {
        return prefs.getFloat(STAIR_WRONG_ANSWER_DB);
    }

    public static void setStaircaseWrongDB(float f) {
        prefs.putFloat(STAIR_WRONG_ANSWER_DB, f);
        prefs.flush();
    }

    public static int getStaircaseMaxTries() {
        return prefs.getInteger(STAIR_MAX_TRIES);
    }

    public static void setStaircaseMaxTires(int i) {
        prefs.putInteger(STAIR_MAX_TRIES, i);
        prefs.flush();
    }

    public static int getStaircaseReversalPoints() {
        return prefs.getInteger(STAIR_REVERSAL_POINTS);
    }

    public static void setStaircaseReversalPoints(int i) {
        prefs.putInteger(STAIR_REVERSAL_POINTS, i);
        prefs.flush();
    }

    public static int getStaircaseReversalsToUse() {
        return prefs.getInteger(STAIR_REVERSALS_TO_USE);
    }

    public static void setStaircaseReversalsToUse(int i) {
        prefs.putInteger(STAIR_REVERSALS_TO_USE, i);
        prefs.flush();
    }


    //Input setters and getters
    public static int getInputKeyLeft() {
        return prefs.getInteger(INPUT_KEY_LEFT);
    }

    public static void setInputKeyLeft(int i) {
        prefs.putInteger(INPUT_KEY_LEFT, i);
        prefs.flush();
    }

    public static int getInputKeyRight() {
        return prefs.getInteger(INPUT_KEY_RIGHT);
    }

    public static void setInputKeyRight(int i) {
        prefs.putInteger(INPUT_KEY_RIGHT, i);
        prefs.flush();
    }

    public static int getInputKeyBack() {
        return prefs.getInteger(INPUT_KEY_BACK);
    }

    public static void setInputKeyBack(int i) {
        prefs.putInteger(INPUT_KEY_BACK, i);
        prefs.flush();
    }

    public static boolean getInputContinuousMode() {
        return prefs.getBoolean(INPUT_CONTINUOUS_MODE);
    }

    public static void setInputContinuousMode(boolean b) {
        prefs.putBoolean(INPUT_CONTINUOUS_MODE, b);
        prefs.flush();
    }

    public static boolean getFullScreenFlag() {
        return true;
    }
}
